// BallsView.cpp : implementation of the CBallsView class
//

#include "stdafx.h"
#include "Balls.h"

#include "BallsDoc.h"
#include "BallsView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CBallsView

IMPLEMENT_DYNCREATE(CBallsView, CView)

BEGIN_MESSAGE_MAP(CBallsView, CView)
	//{{AFX_MSG_MAP(CBallsView)
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_WM_DESTROY()
	ON_WM_ERASEBKGND()
	ON_WM_TIMER()
	ON_WM_KEYDOWN()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBallsView construction/destruction

CBallsView::CBallsView()
{
	// TODO: add construction code here
	m_hDC = NULL;
	m_hRC = NULL;
	m_Timer = 0;
	m_palette = NULL;
}

CBallsView::~CBallsView()
{
}

BOOL CBallsView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs
	cs.style |= (WS_CLIPCHILDREN | WS_CLIPSIBLINGS | CS_OWNDC);

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CBallsView drawing

void CBallsView::OnDraw(CDC* pDC)
{
	CBallsDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here
	if ( NULL != m_hDC && NULL != m_hRC ) 
	{
		VERIFY( wglMakeCurrent( m_hDC, m_hRC ) );

		// do our drawing
		pDoc->DrawScene();

		if ( m_bDoubleBuffer )
			SwapBuffers( m_hDC );
		else
			glFlush();
		
		VERIFY( wglMakeCurrent( NULL, NULL ) );
	}
}

/////////////////////////////////////////////////////////////////////////////
// CBallsView diagnostics

#ifdef _DEBUG
void CBallsView::AssertValid() const
{
	CView::AssertValid();
}

void CBallsView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CBallsDoc* CBallsView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CBallsDoc)));
	return (CBallsDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CBallsView message handlers

int CBallsView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// TODO: Add your specialized creation code here
	
	// Get the DC for the window of this view
	m_hDC = ::GetDC(m_hWnd);

	// set the proper values on the pfd
	PIXELFORMATDESCRIPTOR pfd;
	pfd.nSize = sizeof(PIXELFORMATDESCRIPTOR);
	pfd.nVersion = 1;
	pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL
		| PFD_DOUBLEBUFFER;
	pfd.iPixelType = PFD_TYPE_RGBA;	
	pfd.cColorBits = 24;
	pfd.cDepthBits = 16;
	pfd.cStencilBits = 4;
	pfd.iLayerType = PFD_MAIN_PLANE;

	// Choose a pixel format from the DC
	int nPixelFormat;
	nPixelFormat = ChoosePixelFormat( m_hDC, &pfd );
	 
	if ( 0 == nPixelFormat )
		return -1;

	// Set the pixel format on the DC
	VERIFY( SetPixelFormat( m_hDC, nPixelFormat, &pfd ) );

	// Create a rendering context
	m_hRC = wglCreateContext( m_hDC );

	// Get information about the pfd we actually got
	int nMaxPixelFormat;
	nMaxPixelFormat = DescribePixelFormat( m_hDC, 
			nPixelFormat, sizeof(PIXELFORMATDESCRIPTOR), 
			&pfd );

	// check if we need a palette
	if ( (pfd.dwFlags & PFD_NEED_PALETTE) )
		BuildPalette( &pfd );

	if ( (pfd.dwFlags & PFD_DOUBLEBUFFER) )
		m_bDoubleBuffer = TRUE;
	else
		m_bDoubleBuffer = FALSE;

#if 0
	// hardware acceleration check
	if ( pfd.dwFlags & PFD_GENERIC_FORMAT )
	{
		if ( pfd.dwFlags & PFD_GENERIC_ACCELERATED )
			MessageBox( "MCD accelerated" );
		else
			MessageBox( "SW only path" );
	}
	else if ( pfd.dwFlags & PFD_GENERIC_ACCELERATED )
			MessageBox( "Uncertain path" );
	else
		MessageBox( "ICD accelerated" );
#endif

	if ( NULL != m_hDC && NULL != m_hRC )
	{
		VERIFY( wglMakeCurrent( m_hDC, m_hRC ) );
		SetupScene();
	}

	VERIFY( wglMakeCurrent( NULL, NULL ) );

	/* Setup the timer for stuff to happen */
	m_Timer = SetTimer( 1, 10, NULL );

	Invalidate( FALSE );

	return 0;
}

void CBallsView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	char size[64];

	sprintf(size, "OnSize: %d %d\n", cx, cy );
	
	m_Width = cx;
	m_Height = cy;

	// TODO: Add your message handler code here
	if ( NULL != m_hDC && NULL != m_hRC )
	{
		
		//MessageBox( size );
		VERIFY( wglMakeCurrent( m_hDC, m_hRC ) );
		glViewport( 0, 0, (GLint)cx, (GLint)cy );
	}

	VERIFY( wglMakeCurrent( NULL, NULL ) );
	Invalidate( FALSE );	
}

void CBallsView::OnDestroy() 
{
	
	// TODO: Add your message handler code here
	
	if ( NULL != m_hRC )
	{
		wglMakeCurrent( NULL, NULL );
		wglDeleteContext( m_hRC );
		m_hRC = NULL;
	}

	if ( NULL != m_hDC )
	{
		::ReleaseDC( m_hWnd, m_hDC );
	}

	CView::OnDestroy();
}

BOOL CBallsView::OnEraseBkgnd(CDC* pDC) 
{
	// TODO: Add your message handler code here and/or call default
	// Don't erase the background for us
	return FALSE;
}

void CBallsView::SetupScene()
{
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();

	// setup viewing area
	gluPerspective( 90.0, 1.0, 3.0, 200.0 );

	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();

	gluLookAt( 0.0, 5.0, 60.0, 0.0, 0.0, 0.0, 0.0, 
		1.0, 0.0 );
		
	glPushMatrix();

	CBallsDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pDoc->GLSetupScene();
}

void CBallsView::OnTimer(UINT nIDEvent) 
{
	// TODO: Add your message handler code here and/or call default
	CBallsDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	if ( TRUE == pDoc->OnTimer() )
		Invalidate( FALSE );
						
	CView::OnTimer(nIDEvent);
}

void CBallsView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// TODO: Add your message handler code here and/or call default
	CBallsDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	pDoc->OnKeyDown(nChar, nRepCnt, nFlags);
	CView::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CBallsView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	// This means button 1 and not left button like it says
	CBallsDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	if ( NULL != m_hDC && NULL != m_hRC )
	{
		VERIFY( wglMakeCurrent( m_hDC, m_hRC ) );
		pDoc->OnLButtonDown(nFlags, point);
	}
	CView::OnLButtonDown(nFlags, point);
}

void CBallsView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	CBallsDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	pDoc->OnLButtonUp(nFlags, point);
	CView::OnLButtonUp(nFlags, point);
}

void CBallsView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	CBallsDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	pDoc->OnMouseMove(nFlags, point);
	CView::OnMouseMove(nFlags, point);
}

void CBallsView::BuildPalette(PIXELFORMATDESCRIPTOR * pfd)
{
	LOGPALETTE *pal;
  int i, paletteSize, redMask, greenMask, blueMask;

  paletteSize = 1 << pfd->cColorBits;

  //MessageBox( "Creating palette" );
  pal = (LOGPALETTE *)
    malloc(sizeof(LOGPALETTE) + paletteSize * sizeof(PALETTEENTRY));
  pal->palVersion = 0x300;
  pal->palNumEntries = paletteSize;

  redMask = (1 << pfd->cRedBits) - 1;
  greenMask = (1 << pfd->cGreenBits) - 1;
  blueMask = (1 << pfd->cBlueBits) - 1;

  for (i = 0; i < paletteSize; i++)
    {
      pal->palPalEntry[i].peRed = ((i >> pfd->cRedShift) & redMask);
      pal->palPalEntry[i].peRed = (unsigned char)
	((double) pal->palPalEntry[i].peRed * 255.0 / redMask);
      pal->palPalEntry[i].peGreen = ((i >> pfd->cGreenShift) & greenMask);
      pal->palPalEntry[i].peGreen = (unsigned char)
	((double) pal->palPalEntry[i].peGreen * 255.0 / greenMask);
      pal->palPalEntry[i].peBlue = ((i >> pfd->cBlueShift) & blueMask);
      pal->palPalEntry[i].peBlue = (unsigned char)
	((double) pal->palPalEntry[i].peBlue * 255.0 / blueMask);
      pal->palPalEntry[i].peFlags = 0;
    }

  m_palette = CreatePalette(pal);
  free(pal);

  if (m_palette)
    {
      SelectPalette(m_hDC, m_palette, FALSE);
      RealizePalette(m_hDC);
    }
}
