// BofWonder.cpp: implementation of the CBofWonder class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Balls.h"
#include "BofWonder.h"
#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define BALL_LIST				m_listBase
#define SHADOW_BALL_LIST		m_listBase+1
#define STAND_LIST				m_listBase+10
#define SHADOW_STAND_LIST       m_listBase+11
#define PLANE_LIST				m_listBase+12
#define MIRROR_LIST				m_listBase+13

const GLfloat gravity = (GLfloat)9.81;
const GLfloat radius = (GLfloat)18.0;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBofWonder::CBofWonder()
{
	// Initialize variables
	m_pQuadObjCyl = NULL;
	m_pQuadObjDisk = NULL;
	m_pQuadObjSphere = NULL;
	m_bHaveTextures = FALSE;
	m_bShadowInit = FALSE;
	m_bFront = TRUE;
	m_bObjectsCreated = FALSE;
	m_bRealistic = FALSE;
	m_bMirroring = FALSE;
	m_bTexturing = FALSE; 
	m_bShadowing = FALSE;
	m_bLocating = FALSE;
	m_bRotating = FALSE;
	m_bMoving = FALSE;
	m_bMirrorPass = FALSE;
	m_bNeedInit = TRUE;
	m_bDragging = FALSE;
	m_bLightMoved = FALSE;
	m_nBall = -1;

	m_tx = 0.0f;
	m_ty = 0.0f;
	m_tz = 0.0f;
	
	m_ballColor[0] = 1.0f;
	m_ballColor[1] = 0.0f;
	m_ballColor[2] = 0.0f;
	m_ballColor[3] = 1.0f;

	m_lineColor[0] = 1.0f;
	m_lineColor[1] = 1.0f;
	m_lineColor[2] = 1.0f;
	m_lineColor[3] = 1.0f;

	m_standColor[0] = 0.0f;
	m_standColor[1] = 0.0f;
	m_standColor[2] = 1.0f;
	m_standColor[3] = 1.0f;

	m_shadowColor[0] = 0.0f;
	m_shadowColor[1] = 0.0f;
	m_shadowColor[2] = 0.0f;
	m_shadowColor[3] = 0.4f;

	m_ground[0] = 0.0f;
	m_ground[1] = 1.0f;
	m_ground[2] = 0.0f;
	m_ground[3] = 29.98f;

	m_lightPos[0] = 1.0f;
	m_lightPos[1] = 3.0f;
	m_lightPos[2] = 0.0f;
	m_lightPos[3] = 0.01f;

	m_actualLightPos[0] = m_lightPos[0];
	m_actualLightPos[1] = m_lightPos[1];
	m_actualLightPos[2] = m_lightPos[2];
	m_actualLightPos[3] = m_lightPos[3];

	m_lightRadius = m_lightPos[0];

	m_lightAmb[0] = 0.2f;
	m_lightAmb[1] = 0.2f;
	m_lightAmb[2] = 0.2f;
	m_lightAmb[3] = 1.0f;

	m_lightDiff[0] = 0.8f;
	m_lightDiff[1] = 0.8f;
	m_lightDiff[2] = 0.8f;
	m_lightDiff[3] = 1.0f;

	m_lightSpec[0] = 0.8f;
	m_lightSpec[1] = 0.8f;
	m_lightSpec[2] = 0.8f;
	m_lightSpec[3] = 1.0f;
	
	m_listBase = 0;

	m_pBallTexture = NULL;
	m_pStandTexture = NULL;
	m_pGroundTexture = NULL;

	m_pBallTexture = new CTexture("office.bmp");
	m_pStandTexture = new CTexture("rock.bmp");
	m_pGroundTexture = new CTexture("terra.cotta.bmp");

	if ( NULL != m_pBallTexture && NULL != m_pStandTexture && 
		NULL != m_pGroundTexture )
	{
		if ( m_pBallTexture->m_bValid && m_pStandTexture->m_bValid &&
			m_pGroundTexture->m_bValid )
		{
			m_bHaveTextures = TRUE;
		}
	}

	m_ballTexture = 0;
	m_standTexture = 0;
	m_groundTexture = 0;

	m_iAngle = 0;
	m_iAngle2 = 0;

	m_drawStyle = GLU_FILL;
	m_normals = GLU_SMOOTH;

	InitBalls();

}

CBofWonder::~CBofWonder()
{
	if ( NULL != m_pQuadObjCyl )
		gluDeleteQuadric( m_pQuadObjCyl );
	if ( NULL != m_pQuadObjDisk )
		gluDeleteQuadric( m_pQuadObjDisk );
	if ( NULL != m_pQuadObjSphere )
		gluDeleteQuadric( m_pQuadObjSphere );

	if ( m_bObjectsCreated )
	{
		glDeleteLists( m_listBase, 20 );
	}

	if ( NULL != m_pBallTexture )
		delete m_pBallTexture;
	if ( NULL != m_pStandTexture )
		delete m_pStandTexture;
	if ( NULL != m_pGroundTexture )
		delete m_pGroundTexture;

	if ( 0 != m_ballTexture )
		glDeleteTextures( 1, &m_ballTexture );
	if ( 0 != m_standTexture )
		glDeleteTextures( 1, &m_standTexture );
	if ( 0 != m_groundTexture )
		glDeleteTextures( 1, &m_groundTexture );
	if ( 0 != m_videoTexture )
		glDeleteTextures( 1, &m_videoTexture );
}

void CBofWonder::SetupScene()
{
	glDepthFunc( GL_LEQUAL );
	glClearDepth( 1.0 );
	glClearStencil( 0x0 );
	glClearColor( 0.0, 0.0, 0.0, 1.0 );
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT 
		| GL_STENCIL_BUFFER_BIT );
	
	glLightfv( GL_LIGHT0, GL_POSITION, m_lightPos );
	glLightfv( GL_LIGHT0, GL_AMBIENT, m_lightAmb );
	glLightfv( GL_LIGHT0, GL_DIFFUSE, m_lightDiff );
	glLightfv( GL_LIGHT0, GL_SPECULAR, m_lightSpec );

	glLightModeli( GL_LIGHT_MODEL_LOCAL_VIEWER, 0 );
	glEnable( GL_DEPTH_TEST );
	glEnable( GL_LIGHTING );
	glEnable( GL_LIGHT0 );
	glEnable( GL_COLOR_MATERIAL );
	glEnable( GL_CULL_FACE );
	glCullFace( GL_BACK );
	glBlendFunc( GL_ONE_MINUS_SRC_ALPHA, GL_SRC_ALPHA );
	glColorMaterial( GL_FRONT, GL_AMBIENT_AND_DIFFUSE );

	m_pQuadObjCyl = gluNewQuadric();
	m_pQuadObjDisk = gluNewQuadric();
	m_pQuadObjSphere = gluNewQuadric();
    
    gluQuadricOrientation (m_pQuadObjCyl, GLU_OUTSIDE);
    gluQuadricOrientation (m_pQuadObjDisk, GLU_INSIDE);
    gluQuadricOrientation (m_pQuadObjSphere, GLU_OUTSIDE);

    gluQuadricTexture(m_pQuadObjCyl, GL_TRUE);
    gluQuadricTexture(m_pQuadObjDisk, GL_TRUE);
    gluQuadricTexture(m_pQuadObjSphere, GL_FALSE);

	glGenTextures( 1, &m_ballTexture );
	glGenTextures( 1, &m_standTexture );
	glGenTextures( 1, &m_groundTexture );
	glGenTextures( 1, &m_videoTexture );

}

void CBofWonder::DrawScene()
{

	if ( m_bNeedInit )
		CreateObjects();

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT 
		| GL_STENCIL_BUFFER_BIT );

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	if ( m_bLightMoved )
		glLightfv( GL_LIGHT0, GL_POSITION, m_actualLightPos );
	glTranslatef( m_tx, m_ty, m_tz );
	glRotatef( (GLfloat)m_iAngle2, 0.0, 1.0, 0.0 );
	
	//DrawLight();
	DrawObjects();
	if ( m_bMirroring ) 
		MirrorDraw();
	glPopMatrix();

}

void CBofWonder::CreateObjects()
{
	if ( m_bObjectsCreated )
	{
		glDeleteLists( m_listBase, 20 );
	}
	else
		m_listBase = glGenLists( 20 );

	/* setup the GLU styles	*/
	gluQuadricDrawStyle (m_pQuadObjCyl, m_drawStyle);
    gluQuadricDrawStyle (m_pQuadObjDisk, m_drawStyle);
    gluQuadricDrawStyle (m_pQuadObjSphere, m_drawStyle);
	gluQuadricNormals (m_pQuadObjCyl, m_normals);
    gluQuadricNormals (m_pQuadObjDisk, m_normals);
    gluQuadricNormals (m_pQuadObjSphere, m_normals);

	/* real balls */
    glNewList( BALL_LIST, GL_COMPILE);
       gluSphere(m_pQuadObjSphere, 2.5, 12, 12);
    glEndList();
    /* shadow balls */
    glNewList( SHADOW_BALL_LIST, GL_COMPILE);
       gluSphere(m_pQuadObjSphere, 2.5, 6, 6);
    glEndList();
    /* stand legs */
    glNewList(m_listBase+2, GL_COMPILE);
       gluCylinder(m_pQuadObjCyl, 2.0, 2.0, 50.0, 12, 12);
    glEndList();
    /* short bar */
    glNewList(m_listBase+3, GL_COMPILE);
       gluCylinder(m_pQuadObjCyl, 2.0, 2.0, 20.0, 12, 12);
    glEndList();
    /* Leg */
    glNewList(m_listBase+4, GL_COMPILE);
       gluCylinder(m_pQuadObjCyl, 2.0, 2.0, 34.0, 12, 12);
    glEndList();
    /* Disk for legs */
    glNewList(m_listBase+5, GL_COMPILE);
       gluDisk(m_pQuadObjDisk, 0.0, 2.0, 12, 12);
    glEndList();

    /*shadow stand leg */
    glNewList(m_listBase+6, GL_COMPILE);
       gluCylinder(m_pQuadObjCyl, 2.0, 2.0, 50.0, 4, 4);
    glEndList();
    /*shadow short bar */
    glNewList(m_listBase+7, GL_COMPILE);
       gluCylinder(m_pQuadObjCyl, 2.0, 2.0, 20.0, 4, 4);
    glEndList();
    /*shadow Leg*/
    glNewList(m_listBase+8, GL_COMPILE);
       gluCylinder(m_pQuadObjCyl, 2.0, 2.0, 34.0, 4, 4);
    glEndList();
    /*shadow disk*/
    glNewList(m_listBase+9, GL_COMPILE);
       gluDisk(m_pQuadObjDisk, 0.0, 2.0, 4, 4);
    glEndList();

    /* full stand */
    glNewList( STAND_LIST, GL_COMPILE);
       glRotatef(90.0, 0.0, 1.0, 0.0);
       glTranslatef(10.0, 0.0, -25.0);
       glCallList(m_listBase+2);
       glTranslatef(-20.0, 0.0, 0.0);
       glCallList(m_listBase+2);
       glRotatef(-90.0, 0.0, 1.0, 0.0);
       glTranslatef(0.0, 0.0, -20.0);
       glCallList(m_listBase+3);
       glTranslatef(50.0, 0.0, 0.0);
       glCallList(m_listBase+3);
       glRotatef(90.0, 1.0, 0.0, 0.0);
       glTranslatef(0.0, 20.0, -3.0);
       glCallList(m_listBase+4);
       glCallList(m_listBase+5);
       glTranslatef(0.0, -20.0, 0.0);
       glCallList(m_listBase+4);
       glCallList(m_listBase+5);
       glTranslatef(-50.0, 0.0, 0.0);
       glCallList(m_listBase+4);
       glCallList(m_listBase+5);
       glTranslatef(0.0, 20.0, 0.0);
       glCallList(m_listBase+4);
       glCallList(m_listBase+5);
    glEndList();

    /* shadow stand */
    glNewList( SHADOW_STAND_LIST, GL_COMPILE);
       glRotatef(90.0, 0.0, 1.0, 0.0);
       glTranslatef(10.0, 0.0, -25.0);
       glCallList(m_listBase+6);
       glTranslatef(-20.0, 0.0, 0.0);
       glCallList(m_listBase+6);
       glRotatef(-90.0, 0.0, 1.0, 0.0);
       glTranslatef(0.0, 0.0, -20.0);
       glCallList(m_listBase+7);
       glTranslatef(50.0, 0.0, 0.0);
       glCallList(m_listBase+7);
       glRotatef(90.0, 1.0, 0.0, 0.0);
       glTranslatef(0.0, 20.0, -3.0);
       glCallList(m_listBase+8);
       glCallList(m_listBase+9);
       glTranslatef(0.0, -20.0, 0.0);
       glCallList(m_listBase+8);
       glCallList(m_listBase+9);
       glTranslatef(-50.0, 0.0, 0.0);
       glCallList(m_listBase+8);
       glCallList(m_listBase+9);
       glTranslatef(0.0, 20.0, 0.0);
       glCallList(m_listBase+8);
       glCallList(m_listBase+9);
    glEndList();

    /* create plane */
    glNewList( PLANE_LIST, GL_COMPILE);
       glBegin(GL_POLYGON);
       glNormal3f(0.0, 1.0, 0.0);
       glTexCoord2f(4.0, 4.0);
       glVertex3f(34.0, -30.0, -20.0);
       glTexCoord2f(4.0, 0.0);
       glVertex3f(-34.0, -30.0, -20.0);
       glTexCoord2f(0.0, 0.0);
       glVertex3f(-34.0, -30.0, 20.0);
       glTexCoord2f(0.0, 4.0);
       glVertex3f(34.0, -30.0, 20.0);
       glEnd();
    glEndList();

	
	if( m_bHaveTextures )
	{
		/* texture one */
		glBindTexture(GL_TEXTURE_2D, m_groundTexture);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, 
				GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, 
				GL_NEAREST);
		glTexImage2D(GL_TEXTURE_2D, 0, 3, m_pGroundTexture->m_iWidth, 
				m_pGroundTexture->m_iHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, 
				m_pGroundTexture->m_pData);
		/* texture two */
		glBindTexture(GL_TEXTURE_2D, m_standTexture);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, 
				GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, 
				GL_NEAREST);
		glTexImage2D(GL_TEXTURE_2D, 0, 3, m_pStandTexture->m_iWidth, 
				m_pStandTexture->m_iHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, 
				m_pStandTexture->m_pData);

		/* texture three */
		glBindTexture(GL_TEXTURE_2D, m_ballTexture);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, 
				GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, 
				GL_NEAREST);
		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
		glTexImage2D(GL_TEXTURE_2D, 0, 3, m_pBallTexture->m_iWidth, 
				m_pBallTexture->m_iHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, 
				m_pBallTexture->m_pData);

		/* video texturing replacable stuff */
		glBindTexture(GL_TEXTURE_2D, m_videoTexture);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, 
				GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, 
				GL_NEAREST);
		glTexImage2D(GL_TEXTURE_2D, 0, 3, 512, 
				256, 0, GL_RGB, GL_UNSIGNED_BYTE, 
				NULL);

	}
   
    glNewList( MIRROR_LIST , GL_COMPILE);
      glVertex3f( -80.0, -50.0, -40.0 );
      glVertex3f( 80.0, -50.0, -40.0 );
      glVertex3f( 80.0, 50.0, -40.0 );
      glVertex3f( -80.0, 50.0, -40.0 );
    glEndList();

	m_bObjectsCreated = TRUE;
	m_bNeedInit = FALSE;
}

void CBofWonder::DrawObjects()
{
	GLint k, stop = 0;

    glMatrixMode(GL_MODELVIEW);
    /* We will loop through the drawing twice if we are shadowing */
    if (m_bShadowing && !m_bLocating) 
		stop = 1;
   
    /* draw the ground plane */
    if(m_bTexturing)
	{
		glBindTexture(GL_TEXTURE_2D, m_groundTexture);
		glEnable(GL_TEXTURE_2D);
    }	    
    glPushMatrix(); /* ground plane */
    glColor4fv(m_lineColor);
    if ( m_bMirrorPass ) 
	{
		glStencilFunc(GL_EQUAL, 0x4, 0x7);
		glStencilOp(GL_KEEP, GL_KEEP, GL_INCR);
    }
    else 
	{
		glStencilFunc(GL_EQUAL, 0x0, 0x7);
		glStencilOp(GL_KEEP, GL_KEEP, GL_INCR);
    }
    glEnable(GL_STENCIL_TEST);
    glCallList( PLANE_LIST );    
    glPopMatrix();
    if(m_bTexturing)
		glDisable(GL_TEXTURE_2D); /* turn texturing off */

    for(k = 0; k <= stop; k++) 
	{
		if(k) 
		{ /* if shadow pass */
			glEnable(GL_BLEND);
			glColor4fv(m_shadowColor);
			glStencilOp(GL_KEEP, GL_KEEP, GL_INCR);
			if (m_bMirrorPass)
				glStencilFunc(GL_EQUAL, 0x5, 0x7);
			else 
				glStencilFunc(GL_EQUAL, 0x1, 0x7);
		}
		else 
		{ /* first pass */
			glColor4fv(m_standColor);
			glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
			if (m_bMirrorPass)
				glStencilFunc(GL_EQUAL, 0x4, 0x4);
			else
				glStencilFunc(GL_ALWAYS, 0x0, 0x0);
			if(m_bTexturing)
			{
                glColor4fv(m_lineColor);
				glBindTexture(GL_TEXTURE_2D, m_standTexture);
				glEnable(GL_TEXTURE_2D);
			}
		}
		glPushMatrix();
		if(k)
		{ /* shadow pass */
			ShadowMatrix(m_ground, m_lightPos);
			glGetFloatv(GL_MODELVIEW_MATRIX, 
				(GLfloat *) m_finalMatrix);
			/* use STENCIL to keep track of where */ 
			/* shadows have been drawn  */
			/* don't add the shadow info to the depth buffer */
			glDepthMask(GL_FALSE);
		}	
	
		glLoadName(6);
		if(k)
			glCallList( SHADOW_STAND_LIST );
		else
			glCallList( STAND_LIST );
		glPopMatrix();
	
		if(m_bTexturing)
			glDisable(GL_TEXTURE_2D);
    }
    if(k)
        glDepthMask(GL_TRUE);
    /* Initiate writing to the depth buffer again */
    glDisable(GL_BLEND);
    glDisable(GL_STENCIL_TEST); 
    
    DrawBalls();
}

/* This function takes a ground plane equation and a light position vector
and generates a shadow matrix */
void CBofWonder::ShadowMatrix(GLfloat ground [ 4 ], GLfloat light [ 4 ])
{
    GLfloat shadowMat[16];


    shadowMat[0] = ground[1] * light[1] + ground[2] * light[2];
    shadowMat[4] = 0.0f - light[0] * ground[1];
    shadowMat[8] = 0.0f - light[0] * ground[2];
    shadowMat[12] = 0.0f - light[0] * ground[3];

    shadowMat[1] = 0.0f - ground[0] * light[1];
    shadowMat[5] = ground[0] * light[0] + ground[2] * light[2];
    shadowMat[9] = 0.0f - ground[2] * light[1];
    shadowMat[13] = 0.0f - ground[3] * light[1];

    shadowMat[2] = 0.0f - ground[0] * light[2];
    shadowMat[6] = 0.0f - ground[1] * light[2];
    shadowMat[10] = ground[0] * light[0] + ground[1] * light[1];
    shadowMat[14] = 0.0f - ground[3] * light[2];

    shadowMat[3] = 0.0f;
    shadowMat[7] = 0.0f;
    shadowMat[11] = 0.0f;
    shadowMat[15] = ground[0] * light[0] + ground[1] * light[1] + ground[2] * light[2];

    /* Multiply the shadow matrix by the current Modelview matrix
       after this anything that is drawn before resetting the
       Modelview matrix will appear on the ground plane */
    glMultMatrixf((const GLfloat *) shadowMat);

    m_bShadowInit = TRUE;
}

void CBofWonder::DrawBalls()
{
    GLuint i;
	GLfloat j;
    GLint  k, stop = 0;

    if (m_bShadowing && !m_bLocating)
		stop = 1;

    glEnable(GL_STENCIL_TEST);

    for(k = 0; k <= stop; k++)
	{
		if(k) 
		{ /* shadow pass */
			glEnable(GL_BLEND);
			glDepthMask(GL_FALSE);
			glStencilOp(GL_KEEP, GL_KEEP, GL_INCR);
			if (m_bMirrorPass)
				glStencilFunc(GL_EQUAL, 0x5, 0x7);
			else
				glStencilFunc(GL_EQUAL, 0x1, 0x7);
		}
		else 
		{
			glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
			if (m_bMirrorPass)
				glStencilFunc(GL_EQUAL, 0x4, 0x4);
			else
				glStencilFunc(GL_ALWAYS, 0x0, 0x0);
		}
		/* drawing the balls */
		for(i = 1; i <= 5; i++)
		{
			j = -15.0f + i * 5.0f;
	    
			/* the ball set-up */
			orb[i].x = orb[i].xi - RThetaX(radius, orb[i].theta);
			orb[i].y = -RThetaY(radius, orb[i].theta);
	    
			if(k) 
			{ /* shadow pass */
				glColor4fv(m_shadowColor);
			}
			else
			{
				if (!m_bMirrorPass)
					glLoadName(i);
				glMaterialf(GL_FRONT, GL_SHININESS, 100.0); 
				glColor4fv(m_ballColor);
				if(m_bTexturing)
				{
                    glColor4fv(m_lineColor);
					glBindTexture(GL_TEXTURE_2D, m_ballTexture);
					glEnable(GL_TEXTURE_GEN_T);
					glEnable(GL_TEXTURE_GEN_S);
					glEnable(GL_TEXTURE_2D);
				}
			}
			glPushMatrix();
			if(k)
				glLoadMatrixf(m_finalMatrix);
			glTranslatef(orb[i].x, orb[i].y, orb[i].z);
			if(k)
				glCallList( SHADOW_BALL_LIST );
			else
				glCallList( BALL_LIST );
			glPopMatrix();
	    
			/* drawing the connecting cords */
			glPushMatrix();   
			if(k)
			{
				glColor4fv(m_shadowColor);
				glLoadMatrixf(m_finalMatrix);
			}
			else
			{
				if (!m_bMirrorPass)
					glLoadName(6);
				glMaterialf(GL_FRONT, GL_SHININESS, 0.0);
				glColor4fv(m_lineColor);
				if(m_bTexturing)
				{
					glDisable(GL_TEXTURE_GEN_T);
					glDisable(GL_TEXTURE_GEN_S);
					glDisable(GL_TEXTURE_2D);
				}
			}
			glBegin(GL_LINES);
			glVertex3f(orb[i].x, orb[i].y, orb[i].z);
			glVertex3f(j, -2.0, 10.0);
			glVertex3f(orb[i].x, orb[i].y, orb[i].z);
			glVertex3f(j, -2.0, -10.0);
			glEnd();
			glPopMatrix();
		}
	}
	glDepthMask(GL_TRUE);
	glDisable(GL_STENCIL_TEST);
	glDisable(GL_BLEND);
}

GLfloat CBofWonder::RThetaX(GLfloat r, GLfloat theta)
{
	GLfloat x = 0.0;
	double rad = 0.0;

	rad = theta * ( 0.01745329 );
	x = (GLfloat)(r * sin( rad ));
	
	return x;
}

GLfloat CBofWonder::RThetaY(GLfloat r, GLfloat theta)
{
	GLfloat y = 0.0;
	double rad = 0.0;

	rad = theta * ( 0.01745329 );
	y = (GLfloat)(r * cos( rad ));

	return y;
}

void *CBofWonder::LookupExtension(char * extension, char * function)
{
	char * szBuffer = (char *)glGetString(GL_EXTENSIONS);

	if ( NULL == szBuffer || !strcmp( szBuffer, "" ) )
		return NULL;

	if ( NULL != strstr( szBuffer, extension ) )
	{
		return (wglGetProcAddress( function ));
	}

	return NULL;
	
}

/* function to update the position of the stand */
void CBofWonder::Rotate()
{
	m_iAngle += 1;  /* used to determine if we are at the front or back */
	m_iAngle2 += 1;	/* used to keep track of the motion of the light relative
					to the plane */

	if ( m_iAngle2 >= 360 )
		m_iAngle2 = 0;

	if ( m_iAngle == 90 )
	{
		if ( TRUE == m_bFront )
			m_bFront = FALSE;
		else
			m_bFront = TRUE;
	}

	if ( m_iAngle == 180 )
		m_iAngle = 0;

	/* keep the light at the same place relative to the plane */
	m_lightPos[0] = RThetaY( m_lightRadius, (GLfloat)m_iAngle2 );
	m_lightPos[2] = RThetaX( m_lightRadius, (GLfloat)m_iAngle2 );

}

/* function to update the position of the balls */
void CBofWonder::Motion()
{
	int i, k, col;
	int step;
	int timesteps = 25;
	GLfloat deltat = 1.0f/(GLfloat)timesteps;
	GLfloat temp;

    
	for (step = 0; step < timesteps; step++)
	{
		/* check for collisions */

		for (i = 1; i < 5; i++)
		{
			k = 5 + 1 - i;
			/* between ball[i] and ball[i+1] */
			if (col = CheckCollision(orb[i].theta, orb[i+1].theta, orb[i].v_theta, orb[i+1].v_theta))
			{
				switch(col) 
				{
					case 1:
					case 2:
					case 3:
						temp = orb[i].v_theta;
						orb[i].v_theta = orb[i+1].v_theta;
						orb[i+1].v_theta = temp;
						break;
					case 4:
						orb[i+1].v_theta = orb[i].v_theta;
						orb[i].v_theta = 0.0;
						if( FALSE == m_bRealistic )
							orb[i].theta = 0.0;
						break;
					case 5:
						orb[i].v_theta = orb[i+1].v_theta;
						orb[i+1].v_theta = 0.0;
						if( FALSE == m_bRealistic )
							orb[i+1].theta = 0.0;
						break;
					default:
						break;
				}
			}

			/* between ball[k] and ball[k-1] */
			if (col = CheckCollision(orb[k-1].theta, orb[k].theta, orb[k-1].v_theta, orb[k].v_theta))
			{
				switch(col) 
				{
					case 1:
					case 2:
					case 3:
						temp = orb[k].v_theta;
						orb[k].v_theta = orb[k-1].v_theta;
						orb[k-1].v_theta = temp;
						break;
					case 4:
						orb[k].v_theta = orb[k-1].v_theta;
						orb[k-1].v_theta = 0.0;
						if( FALSE == m_bRealistic )
							orb[k-1].theta = 0.0;
						break;
					case 5:
						orb[k-1].v_theta = orb[k].v_theta;
						orb[k].v_theta = 0.0;
						if( FALSE == m_bRealistic )
							orb[k].theta = 0.0;
						break;
					default:
						break;
				}
			}
		}
		for (i = 1; i <= 5; i++)
		{
			/* equations of motion */
			orb[i].a_theta = (GLfloat)(gravity * sin((orb[i].theta * 0.01745329)) / radius);
			orb[i].v_theta = (orb[i].a_theta * deltat) + orb[i].v_theta;
			orb[i].theta = orb[i].theta - orb[i].v_theta * deltat;
		}
	}
}

GLint CBofWonder::CheckCollision(GLfloat theta1, GLfloat theta2, GLfloat v1, GLfloat v2)
{
	GLint colide = 0;

	if ( theta1 - theta2 < 0.001 )
	{
		if (v1 > 0.0 && v2 < 0.0)
		{ /* coming together */
			colide = 1;
            /* noise(); */
			return colide; 
		}
		if (v1 > 0.0 && v2 > 0.0 && v1 > v2)
		{ /* both going right */
			colide = 2;
			return colide; 
		}
		if (v1 < 0.0 && v2 < 0.0 && v1 > v2)
		{ /* both going left */
			colide = 3;
			return colide; 
		}
		if (v1 > 0.0 && v2 == 0.0)
		{ /* ball on right is stopped */
			colide = 4;
			return colide; 
		}
		if (v1 == 0.0 && v2 < 0.0)
		{ /* ball on left is stopped */
			colide = 5;
			return colide; 
		}
		else 
		{   /* the unexpected case, maybe they aren't really colliding */
			colide = 6;
			return colide; 
		}
    }
    else
		colide = 0;

    return colide;
}

BOOL CBofWonder::OnTimer()
{
	if ( TRUE == m_bRotating )
		Rotate();
	if ( TRUE == m_bMoving )
		Motion();

	if ( TRUE == m_bRotating || TRUE == m_bMoving )
		return TRUE;

	return FALSE;
}

void CBofWonder::MirrorDraw()
{

    /* STENCIL BUFFER USAGE 
       Number  bits   meaning 
         0     000    empty
         1     001    ground plane  => Ok to draw shadow here
         2     010    shadow w/ground plane => shadow already drawn here
         3     011    Mask value
         4     100    Mirror => Ok to draw mirrored images here
         5     101    Mirror w/ground plane => Ok to draw mirrored shadow images here
         6     110    shadow, Mirror w/ground plane => shadow already drawn here 
         7     111    Mask value
     */
	m_bMirrorPass = TRUE;

    /* Draw the mirror */
    glPushMatrix();
    glRotatef( (GLfloat)-m_iAngle2, 0.0, 1.0, 0.0 );
    glColor3f(0.5, 0.5, 0.5 );
    
    glStencilFunc(GL_ALWAYS, 0x4, 0x7);
    glStencilOp(GL_REPLACE, GL_KEEP, GL_REPLACE);
    glEnable(GL_STENCIL_TEST);
    glDepthMask( GL_FALSE );
    glColorMask( GL_FALSE, GL_FALSE, GL_FALSE, 
		 GL_FALSE );
    glBegin(GL_QUADS);
    glCallList( MIRROR_LIST );
    glEnd();
	
    glColorMask( GL_TRUE, GL_TRUE, GL_TRUE,
		 GL_TRUE );
    glDepthMask( GL_TRUE );

    /* redraw the scene in mirrored mode */
    
    /* draw it */    
    glPushMatrix();
    glTranslatef( 0.0, 0.0, -80.0 );
    glRotatef( (GLfloat)-m_iAngle2, 0.0, 1.0, 0.0 );
    /* move the light position */
    m_lightPos[0] = (GLfloat)RThetaY( 1.0, (GLfloat)-m_iAngle2 );
    m_lightPos[2] = (GLfloat)RThetaX( 1.0, (GLfloat)-m_iAngle2 );

    DrawObjects();
    
    /* move the light back */
    m_lightPos[0] = (GLfloat)RThetaY( 1.0, (GLfloat)m_iAngle2 );
    m_lightPos[2] = (GLfloat)RThetaX( 1.0, (GLfloat)m_iAngle2 );
    glPopMatrix();

    glDisable(GL_STENCIL_TEST);
    glDisable(GL_BLEND);
    glDisable(GL_TEXTURE_2D);
    
    /* draw the mirror outline */
    glColor4d(0.8, 0.8, 0.8, 1.0);
    glBegin(GL_LINE_LOOP);
    glCallList( MIRROR_LIST );
    glEnd();

    /* draw the beautiful video textured background */
    glStencilFunc(GL_EQUAL, 0x4, 0x7);
    glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
    glEnable(GL_STENCIL_TEST);
    glColor4f(0.0, 0.0, 0.0, 1.0);
#if 0
    if (m_bTexturing ) 
	{
	    glBindTextureEXT(GL_TEXTURE_2D, m_videoTexture);
		glEnable(GL_TEXTURE_2D);
    }
#endif
    glBegin(GL_POLYGON);
    glNormal3f( 0.0, 0.0, 1.0 );
    glTexCoord2f( 1.0, 1.0 );
    glVertex3f( 150.0, 100.0, -120.0 );
    glNormal3f( 0.0, 0.0, 1.0 );
    glTexCoord2f( 0.0, 1.0 );
    glVertex3f( -150.0, 100.0, -120.0 );
    glNormal3f( 0.0, 0.0, 1.0 );
    glTexCoord2f( 0.0, 0.0 );
    glVertex3f( -150.0, -100.0, -120.0 );
    glNormal3f( 0.0, 0.0, 1.0 );
    glTexCoord2f( 1.0, 0.0 );
    glVertex3f( 150.0, -100.0, -120.0 );
    glEnd();
    if (m_bTexturing)
		glDisable(GL_TEXTURE_2D);
    glDisable(GL_STENCIL_TEST);

    glPopMatrix();

	m_bMirrorPass = FALSE;
}

void CBofWonder::OnPositionReset()
{
	orb[1].theta = 0.0;
    orb[2].theta = 0.0;
    orb[3].theta = 0.0;
    orb[4].theta = 0.0;
    orb[5].theta = 0.0;

	InitBalls();
}

void CBofWonder::OnPosition1()
{
	orb[1].theta = 40.0;
    orb[2].theta = 0.0;
    orb[3].theta = 0.0;
    orb[4].theta = 0.0;
    orb[5].theta = 0.0;

	InitBalls();

}

void CBofWonder::OnPosition2()
{
	orb[1].theta = 40.0;
    orb[2].theta = 40.0;
    orb[3].theta = 0.0;
    orb[4].theta = 0.0;
    orb[5].theta = 0.0;

	InitBalls();
}

void CBofWonder::OnPosition3()
{
	orb[1].theta = 40.0;
    orb[2].theta = 40.0;
    orb[3].theta = 40.0;
    orb[4].theta = 0.0;
    orb[5].theta = 0.0;

	InitBalls();
}

void CBofWonder::OnPosition4()
{
	orb[1].theta = 40.0;
    orb[2].theta = 40.0;
    orb[3].theta = 40.0;
    orb[4].theta = 40.0;
    orb[5].theta = 0.0;

	InitBalls();
}

void CBofWonder::OnPosition5()
{
	orb[1].theta = 40.0;
    orb[2].theta = 40.0;
    orb[3].theta = 40.0;
    orb[4].theta = 40.0;
    orb[5].theta = 40.0;

	InitBalls();
}

void CBofWonder::InitBalls()
{
	orb[1].InitValues( 1 );
	orb[2].InitValues( 2 );
	orb[3].InitValues( 3 );
	orb[4].InitValues( 4 );
	orb[5].InitValues( 5 );
}

BOOL CBofWonder::SelectBall(GLint x, GLint y)
{
	#define BUFSIZE 512

    GLuint selectBuf[BUFSIZE];
    GLint viewport[4];
    GLint hits, i;
	GLuint j;
    GLuint *ptr, names, ball;
    GLenum errorCode;
    GLsizei width, height;

	m_bLocating = TRUE;

    /* buffer to hold the selection information */
    glSelectBuffer(BUFSIZE, selectBuf);

    glGetIntegerv( GL_VIEWPORT, viewport );
	viewport[0] = 0;
	viewport[1] = 0;
    width = viewport[2];
	height = viewport[3];
    glRenderMode(GL_SELECT); /* change to render mode for selection */
    glInitNames();
    glPushName(-1);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
    /* setting the picking area of the window in pixels, note that X labels
       the y-axis from the top down and opengl goes from bottom up. */
    gluPickMatrix(x, (height - y), 2.0, 2.0, viewport);
    gluPerspective(90.0, 1.0, 3.0, 200.0);
    gluLookAt(0.0, 5.0, 60.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
    glMatrixMode(GL_MODELVIEW);
    /* draw the small area of the scene we just set up and see what appears
       the things that are in the area are called hits */
    DrawScene();
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

    hits = glRenderMode(GL_RENDER);

    while((errorCode = glGetError()) != GL_NO_ERROR)
        fprintf(stderr, "GL error: %s\n", gluErrorString(errorCode));
    m_bLocating = FALSE;

    ptr = (GLuint *) selectBuf;

    if (hits > 0) 
	{
		/*printf("Hits:%d\n", hits);*/
        for (i = 0; i < hits; i++) 
		{
			names = *ptr;
			ptr++;
			/*printf("Hit[%d]: contains %d names\n", i, names);*/
			/*printf("Minimum:0x%x ", *ptr);*/
			ptr++;
			/*printf("Maximum:0x%x\n", *ptr);*/
			ptr++;
			/*printf("Names in bottom up order\n");*/
			for (j = 0; j < names; j++) 
			{
				if (6 != *ptr)
					ball = *ptr;
				/*printf("Name[%d]:%d\n", j, ball);*/
				ptr++;
			}
		}
        if (ball < 6)
		{
            m_nBall = (GLint)ball;
			m_x = x;
			m_y = y;
			return TRUE;
		}
		else
		{
			m_nBall = -1;
			return FALSE;
		}
    }

	return FALSE;
}

void CBofWonder::MoveBall(GLint x, GLint y)
{
	GLint delta_x;
	GLint direction = 0; /* Direction of travel 1 = left, 2 = right, 0 = not moving */
	GLint approach = 0;

	if ( -1 == m_nBall )
		return;

	/* set the velocity and accel of all the balls to zero */
    for (int i = 1; i <= 5; i++) {
        orb[i].v_theta = 0.0;
        orb[i].a_theta = 0.0;
    }

	delta_x = (int)(0.6f * ( m_x - x ));
	if ( !m_bFront )
		delta_x = -1 * delta_x;

	orb[m_nBall].theta = orb[m_nBall].theta + delta_x;

	if ( delta_x > 0 )
		direction = 1;
	else if ( delta_x < 0 )
		direction = 2;
	else
		direction = 0;

	if ( direction == 1 )
		approach = m_nBall -1;
	if ( direction == 2 ) 
		approach = m_nBall + 1;

	if ( approach != 0 && approach != 6 ) {
		if ( approach < m_nBall )
		{
			if (orb[m_nBall].theta >= orb[approach].theta)
				orb[m_nBall].theta = orb[approach].theta;
		}
		else 
		{
			if (orb[m_nBall].theta <= orb[approach].theta)
				orb[m_nBall].theta = orb[approach].theta;
		}
	}
		    
    if(orb[m_nBall].theta >= 83.4f) 
		orb[m_nBall].theta = 83.4f; 
    if(orb[m_nBall].theta <= -83.4f)
		orb[m_nBall].theta = -83.4f;
		
	orb[m_nBall].x = RThetaX(radius, orb[m_nBall].theta);

	m_x = x;
	m_y = y;

}

void CBofWonder::DrawLight()
{
   glDisable(GL_LIGHTING);
   glColor3f(0.0, 1.0, 0.0);
   glBegin(GL_LINES);
   glVertex3f(0.0, 0.0, 0.0);
   glVertex3f(m_lightPos[0]/m_lightPos[3], m_lightPos[1]/m_lightPos[3], 
			  m_lightPos[2]/m_lightPos[3]);
   glEnd();
   glEnable(GL_LIGHTING);
}
