// BofWonder.h: interface for the CBofWonder class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_BOFWONDER_H__0846FD2E_C99C_11D1_AA63_00C04FB176AD__INCLUDED_)
#define AFX_BOFWONDER_H__0846FD2E_C99C_11D1_AA63_00C04FB176AD__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "Texture.h"

class CBall 
{
public:
	GLfloat x;         /* x position */
    GLfloat y;         /* y position */
    GLfloat z;         /* z position */
    GLfloat xi;        /* initial x position */
    GLfloat theta;     /* angular rotation */   
    GLfloat v_theta;   /* angular velocity */
    GLfloat a_theta;   /* angular acceleration */

	CBall() {
		x = 0.0;
		y = -20.0;
		z = 0.0;
		xi = 0.0;
		theta = 0.0;
		v_theta = 0.0;
		a_theta = 0.0;
	}
	~CBall() {}

	void InitValues( int val ) {
		GLfloat j = (GLfloat)(-15.0 + val * 5.0);
		x = j;
		xi = j;
		y = -20.0;
		z = 0.0;
		v_theta = 0.0;
		a_theta = 0.0;
	}
};

class CBofWonder  
{
public:
	void DrawLight();
	GLint m_y;
	GLint m_x;
	BOOL m_bDragging;
	GLint m_nBall;
	void MoveBall( GLint x, GLint y);
	BOOL SelectBall( GLint x, GLint y);
	void OnPosition5();
	void OnPosition4();
	void OnPosition3();
	void OnPosition2();
	void OnPosition1();
	void OnPositionReset();
	BOOL m_bMirrorPass;
	void MirrorDraw();
	BOOL OnTimer();
	void Motion();
	void Rotate();
	int m_iAngle2;
	int m_iAngle;
	void *LookupExtension( char *extension, char *function );
	void DrawBalls();
	void DrawObjects();
	GLuint m_videoTexture;
	void DrawScene();
	void SetupScene();

	// glu objects
	GLUquadricObj *m_pQuadObjCyl;
	GLUquadricObj *m_pQuadObjDisk;
	GLUquadricObj *m_pQuadObjSphere;
	// state variables
	BOOL m_bHaveTextures;
	BOOL m_bShadowInit;
	BOOL m_bFront;
	BOOL m_bObjectsCreated;
	BOOL m_bRealistic;
	BOOL m_bMirroring;
	BOOL m_bTexturing;
	BOOL m_bShadowing;
	BOOL m_bLocating;
	BOOL m_bRotating;
	BOOL m_bMoving;
	BOOL m_bNeedInit;
	BOOL m_bLightMoved;
	GLenum m_normals;
	GLenum m_drawStyle;
	GLfloat m_tx;
	GLfloat m_ty;
	GLfloat m_tz;
	GLfloat m_lightPos[4];
	GLfloat m_actualLightPos[4];
	GLfloat m_lightRadius;

	CBofWonder();
	virtual ~CBofWonder();

protected:
	void InitBalls();
	GLint CheckCollision(GLfloat theta1, GLfloat theta2, GLfloat v1, GLfloat v2 );
	GLfloat RThetaY( GLfloat r, GLfloat theta );
	GLfloat RThetaX( GLfloat r, GLfloat theta );
	void ShadowMatrix( GLfloat ground[4], GLfloat light[4]);
	void CreateObjects();
	// color constants
	GLfloat m_ballColor[4];
	GLfloat m_lineColor[4];
	GLfloat m_standColor[4];
	GLfloat m_shadowColor[4];
	
	GLfloat m_ground[4];
	GLfloat m_finalMatrix[16];
	GLfloat m_lightAmb[4];
	GLfloat m_lightDiff[4];
	GLfloat m_lightSpec[4];
	GLuint m_listBase;

	CTexture *m_pBallTexture;
	CTexture *m_pStandTexture;
	CTexture *m_pGroundTexture;
	GLuint m_ballTexture;
	GLuint m_standTexture;
	GLuint m_groundTexture;

	CBall orb[6];
};

#endif // !defined(AFX_BOFWONDER_H__0846FD2E_C99C_11D1_AA63_00C04FB176AD__INCLUDED_)
