#include <stdio.h>
#include <io.h>
#include <fcntl.h>
#include "Simple.h"

int 			winWidth, winHeight; // Window width and height
HDC			    dc = NULL;					 // Device Context
HGLRC			rc = NULL;					 // OpenGL renering context
HWND			wnd;				 // Window handle
BOOL            doubleBuffered;      // Is the window doublebuffered
HPALETTE        palette;             // windows palette

/////////////////////////////////////////////////////////////////////////////
// Draw
//
// OpenGL rendering commands go here.
//
/////////////////////////////////////////////////////////////////////////////
void Draw()
{
	wglMakeCurrent(dc, rc);
	
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	
	glMatrixMode( GL_MODELVIEW );
	
	glPushMatrix();
	glTranslatef( 0.0, 0.0, -250.0 );

	glColor4f( 1.0, 0.0, 0.0, 1.0 ); 
	glBegin( GL_QUADS );
	glVertex2f( -20.0f, -20.0f );
	glVertex2f( -20.0f,  20.0f );
	glVertex2f( 20.0f, 20.0f );
	glVertex2f( 20.0f, -20.0f );
	glEnd();

	glPopMatrix();
	
	if ( doubleBuffered )
		SwapBuffers(dc);
	else
		glFlush();
	
	ValidateRect(wnd, NULL);
}


/////////////////////////////////////////////////////////////////////////////
// Init
//
// Choose pixel format, create palette if necessary, initialize OpenGL
// viewing and rendering state.
//
/////////////////////////////////////////////////////////////////////////////
void Init()
{
	PIXELFORMATDESCRIPTOR	pfd;
	PIXELFORMATDESCRIPTOR	tempPfd;
	int	pixelFormat;
	
	pfd.nSize =				sizeof(PIXELFORMATDESCRIPTOR);
	pfd.nVersion = 			1;
	pfd.dwFlags =			PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL |
							PFD_DOUBLEBUFFER;
	pfd.iPixelType =        PFD_TYPE_RGBA;
	pfd.cColorBits =		24;
	pfd.cRedBits =			0;
	pfd.cRedShift =			0;
	pfd.cGreenBits =		0;
	pfd.cGreenShift =		0;
	pfd.cBlueBits =			0;
	pfd.cBlueShift =		0;
	pfd.cAlphaBits =		0;
	pfd.cAlphaShift =		0;
	pfd.cAccumBits =		0;
	pfd.cAccumRedBits =		0;
	pfd.cAccumGreenBits =	0;
	pfd.cAccumBlueBits =	0;
	pfd.cAccumAlphaBits =	0;
	pfd.cDepthBits =		0;
	pfd.cStencilBits =		0;
	pfd.cAuxBuffers =		0;
	pfd.iLayerType =		PFD_MAIN_PLANE;
	pfd.bReserved =			0;
	pfd.dwLayerMask =		0;
	pfd.dwVisibleMask =		0;
	pfd.dwDamageMask =		0;
	
	
	dc = GetDC(wnd);
	pixelFormat = ChoosePixelFormat(dc, &pfd);

	DescribePixelFormat(dc, pixelFormat,
						sizeof(PIXELFORMATDESCRIPTOR), &tempPfd);
	
	if ( tempPfd.iPixelType != PFD_TYPE_RGBA )
	{
		MessageBox(	wnd, "Couldn't get an RGB pfd", NULL, MB_OK );
		Quit();
	}

	if ( tempPfd.cColorBits < 24 )
		MessageBox( wnd, "Didn't get a 24 bit window the app might not look right.", NULL, MB_OK );

	if ( tempPfd.dwFlags & PFD_DOUBLEBUFFER )
		doubleBuffered = TRUE;
	else
		doubleBuffered = FALSE;

	if(SetPixelFormat(dc, pixelFormat, &pfd) == FALSE)
		exit(1);

	rc = wglCreateContext(dc);
	wglMakeCurrent(dc, rc);

	SetupScene();

	wglMakeCurrent( NULL, NULL );
}

/////////////////////////////////////////////////////////////////////////////
// SetupScene
//
// Set up the OpenGL state for the scene
//
/////////////////////////////////////////////////////////////////////////////
void SetupScene()
{

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	gluPerspective( 75.0, 1.0, 0.3, 400.0 );

	glClearColor( 0.0, 0.0, 0.0, 1.0 );
}

/////////////////////////////////////////////////////////////////////////////
// Quit
//
// Do Some OpenGL context cleanup and post a quit message when the app
// exits.
//
/////////////////////////////////////////////////////////////////////////////
void Quit()
{
	wglMakeCurrent(NULL, NULL);
	wglDeleteContext(rc);
	ReleaseDC(wnd, dc);
	PostQuitMessage(0);
}

/////////////////////////////////////////////////////////////////////////////
// Resize
//
// Adjusts viewport and vs.projection when window is resized.
//
/////////////////////////////////////////////////////////////////////////////
void Resize()
{
	wglMakeCurrent(dc, rc);

	glViewport(0,0,winWidth, winHeight);
	InvalidateRect(wnd, NULL, FALSE);
	
}

/////////////////////////////////////////////////////////////////////////////
// Resize
//
// Adjusts viewport and vs.projection when window is resized.
//
/////////////////////////////////////////////////////////////////////////////
void BuildPalette( PIXELFORMATDESCRIPTOR *pfd )
{
	LOGPALETTE *pal;
	int i, paletteSize, redMask, greenMask, blueMask;
	
	paletteSize = 1 << pfd->cColorBits;
	
	pal = (LOGPALETTE *)
		malloc(sizeof(LOGPALETTE) + paletteSize * sizeof(PALETTEENTRY));
	pal->palVersion = 0x300;
	pal->palNumEntries = paletteSize;
	
	redMask = (1 << pfd->cRedBits) - 1;
	greenMask = (1 << pfd->cGreenBits) - 1;
	blueMask = (1 << pfd->cBlueBits) - 1;
	
	for (i = 0; i < paletteSize; i++)
    {
		pal->palPalEntry[i].peRed = ((i >> pfd->cRedShift) & redMask);
		pal->palPalEntry[i].peRed = (unsigned char)
			((double) pal->palPalEntry[i].peRed * 255.0 / redMask);
		pal->palPalEntry[i].peGreen = ((i >> pfd->cGreenShift) & greenMask);
		pal->palPalEntry[i].peGreen = (unsigned char)
			((double) pal->palPalEntry[i].peGreen * 255.0 / greenMask);
		pal->palPalEntry[i].peBlue = ((i >> pfd->cBlueShift) & blueMask);
		pal->palPalEntry[i].peBlue = (unsigned char)
			((double) pal->palPalEntry[i].peBlue * 255.0 / blueMask);
		pal->palPalEntry[i].peFlags = 0;
    }
	
	palette = CreatePalette(pal);
	free(pal);
	
	if (palette)
    {
		SelectPalette(dc, palette, FALSE);
		RealizePalette(dc);
    }
}

/////////////////////////////////////////////////////////////////////////////
// WndProc
//
// Route/process windows events.
//
/////////////////////////////////////////////////////////////////////////////
LRESULT CALLBACK WndProc(HWND _wnd, UINT _msg, WPARAM _wParam, LPARAM _lParam)
{
	wnd = _wnd;
	switch(_msg)
	{
	case WM_CREATE:
		Init();
		return 0;
	case WM_PAINT:
		PAINTSTRUCT ps;
		BeginPaint(wnd, &ps);
		if(rc)
			Draw();
		EndPaint(wnd, &ps);
		return 0;
	case WM_SIZE:
		winWidth = LOWORD(_lParam); 
		winHeight = HIWORD(_lParam);
		Resize();
		return 0;
	case WM_ERASEBKGND:
		return 0;
	case WM_SYSCOMMAND:
		if (_wParam == SC_CLOSE)
		{
			Quit();
			return 0;
		}
		break;
	case WM_DESTROY:
		Quit();
		return 0;
	case WM_KEYDOWN:
		switch (_wParam) 
		{
		case VK_ESCAPE:
			Quit();
			return 0;
		}
		break;
	}
	
	return DefWindowProc (wnd, _msg, _wParam, _lParam) ;
}


/////////////////////////////////////////////////////////////////////////////
// WinMain
//
// Win32 required main routine.  Create window.  Poll for windows events.
//
/////////////////////////////////////////////////////////////////////////////
int APIENTRY WinMain(HINSTANCE _instance, HINSTANCE _prevInst, LPSTR _cmdLine,
					 int _cmdShow)
{
	MSG msg ;
	WNDCLASSEX wndClass;
	char *className = "OpenGL";
	char *windowName = "Simple OpenGL Program";
	
	winWidth = winHeight = 500;
	
	wndClass.cbSize        = sizeof (WNDCLASSEX) ;
	wndClass.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wndClass.lpfnWndProc   = WndProc;
	wndClass.cbClsExtra    = 0 ;
	wndClass.cbWndExtra    = 0 ;
	wndClass.hInstance     = _instance ;
	wndClass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
	wndClass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
	wndClass.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH);
	wndClass.lpszMenuName  = NULL ;
	wndClass.lpszClassName = className ;
	wndClass.hIconSm       = LoadIcon (NULL, IDI_APPLICATION) ;
	
	RegisterClassEx (&wndClass) ;
	
	wnd = CreateWindow(className, windowName,
		WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
		100,					// initial x position
		100,					// initial y position
		winWidth,				// winWidth
		winHeight,				// winHeight
		NULL,  	  			// parent window handle
		(HMENU) NULL,		// window menu handle
		_instance,			// program instance handle
		NULL) ;										
	
	ShowWindow(wnd, _cmdShow);
	UpdateWindow(wnd);
	
	while (GetMessage (&msg, NULL, 0, 0))
	{
		TranslateMessage (&msg) ;
		DispatchMessage (&msg) ;
	}
	return msg.wParam ;
}

