// Texture.cpp: implementation of the CTexture class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Balls.h"
#include "Texture.h"
#include <string.h>
#include <stdlib.h>
#include <malloc.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTexture::CTexture( char * filename )
{

	m_iWidth = 0;
	m_iHeight = 0;
	m_pData = NULL;
	m_pFileName = NULL;
	m_bValid = FALSE;
	m_pFileName = _strdup( filename );
    m_bValid = LoadDIBitmap();
	
}

CTexture::~CTexture()
{

	if ( NULL != m_pFileName )
		free( m_pFileName );

	if ( NULL != m_pData )
		free( m_pData );
}

BOOL CTexture::LoadDIBitmap()
{
	FILE				*fp;		/* Open file pointer */
	void				*bits;		/* Bitmap pixel bits */
	long				bitsize,	/* Size of bitmap */
						infosize;	/* Size of header information */
	BITMAPFILEHEADER	header;		/* File header */
	BITMAPINFO			*info;      /* Bitmap information */

	/*
	* Try opening the file; use "rb" mode to read this *binary* file.
	*/

	if ((fp = fopen(m_pFileName, "rb")) == NULL)
		return FALSE;

	/*
	* Read the file header and any following bitmap information...
	*/

	if (fread(&header, sizeof(BITMAPFILEHEADER), 1, fp) < 1)
	{
		/*
		* Couldn't read the file header - return FALSE...
		*/

		fclose(fp);
		return FALSE;
	}

	if (header.bfType != 'MB')	/* Check for BM reversed... */
	{
		/*
		* Not a bitmap file - return FALSE...
		*/

		fclose(fp);
		return FALSE;
	}

	infosize = header.bfOffBits - sizeof(BITMAPFILEHEADER);
	if ((info = (BITMAPINFO *)malloc(infosize)) == NULL)
	{
		/*
		* Couldn't allocate memory for bitmap info - return FALSE...
		*/

		fclose(fp);
		return FALSE;
	}

	if ((long)fread(info, 1, infosize, fp) < infosize)
	{
		/*
		* Couldn't read the bitmap header - return FALSE...
		*/	

		free(info);
		fclose(fp);
		return FALSE;
	}

	/*
	* Now that we have all the header info read in, allocate memory for the
	* bitmap and read *it* in...
	*/

	if ((bitsize = info->bmiHeader.biSizeImage) == 0)
		bitsize = (info->bmiHeader.biWidth *
               info->bmiHeader.biBitCount + 7) / 8 *
  	      abs(info->bmiHeader.biHeight);

	if ((bits = malloc(bitsize)) == NULL)
	{
		/*
		 * Couldn't allocate memory - return FALSE!
		*/

		free(info);
		fclose(fp);
		return FALSE;
	}

	if ((long)fread(bits, 1, bitsize, fp) < bitsize)
	{
		/*
		* Couldn't read bitmap - free memory and return FALSE!
		*/

		free(info);
		free(bits);
		fclose(fp);
		return FALSE;
	}

	/*
	* OK, everything went fine - return the allocated bitmap...
	*/

	fclose(fp);

	BOOL bVal = ConvertRGB( info, bits );
	free(info);
	free(bits);

	return bVal;
}

BOOL CTexture::ConvertRGB(BITMAPINFO * info, void * bits)
{
	int		i, j,			/* Looping vars */
  			bitsize,		/* Total size of bitmap */
			width;			/* Aligned width of bitmap */
	GLubyte	*newbits;		/* New RGB bits */
	GLubyte	*from, *to;		/* RGB looping vars */



	/*
	* Allocate memory for the RGB bitmap...
	*/

	width   = 3 * info->bmiHeader.biWidth;
	width   = (width + 3) & ~3;	
	bitsize = width * info->bmiHeader.biHeight;
	if ((newbits = (GLubyte *)calloc(bitsize, 1)) == NULL)
		return FALSE;

	m_iWidth = info->bmiHeader.biWidth;
	m_iHeight = info->bmiHeader.biHeight;
	m_pData = newbits;
	/*
	* Copy the original bitmap to the new array, converting as necessary.
	*/

	switch (info->bmiHeader.biCompression)
	{
		case BI_RGB :
			if (info->bmiHeader.biBitCount == 24)
			{
			/*
			* Swap red & blue in a 24-bit image...
			*/

			for (i = 0; i < info->bmiHeader.biHeight; i ++)
				for (j = 0, from = ((GLubyte *)bits) + i * width,
					to = newbits + i * width;
					j < info->bmiHeader.biWidth;
					j ++, from += 3, to += 3)
				{
					to[0] = from[2];
					to[1] = from[1];
					to[2] = from[0];
				}
			}
			break;
		case BI_RLE4 :
		case BI_RLE8 :
		case BI_BITFIELDS :
			break;
	}

	return TRUE;
}
