/*
 * Copyright (c) 1998-99 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdio.h>
#include <windows.h>
#include <iostream.h>
#include "handy.h"
#include "jinfo.h"
#include "fileutils.h"

__declspec(dllexport)
fileHeader_t *fileCreate(char *fname, int direct) 
{
    fileHeader_t *f;
    DWORD flags = FILE_ATTRIBUTE_NORMAL;

    NC (f = (fileHeader_t *)VirtualAlloc(NULL, sizeof(fileHeader_t), MEM_COMMIT, PAGE_READWRITE));

    memset(f, 0x0, sizeof(fileHeader_t));

    if (direct) {
        flags = FILE_FLAG_NO_BUFFERING; 
    }

    f->fd = CreateFile(fname, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, flags, NULL);
    ERR((f->fd == INVALID_HANDLE_VALUE), "Can't create new file");

    SetFilePointer(f->fd, 0, NULL, FILE_BEGIN);

    f->headerSize = sizeof(fileHeader_t);

    ZC (WriteFile(f->fd, f, sizeof(fileHeader_t), &flags, NULL));
    return f;
}


__declspec(dllexport)
void fileInit(char *fname, fileHeader_t *f, int direct) 
{
    HANDLE fd;
    DWORD flags = FILE_ATTRIBUTE_NORMAL;

    if (direct) {
        flags = FILE_FLAG_NO_BUFFERING; 
    }

    fd = CreateFile(fname, GENERIC_READ, 0, NULL, OPEN_EXISTING, flags, NULL);
    ERR((fd == INVALID_HANDLE_VALUE), "Can't open existing file");

    SetFilePointer(fd, 0, NULL, FILE_BEGIN);

    ZC (ReadFile(fd, f, sizeof(fileHeader_t), &flags, NULL));
    f->fd = fd;
}


__declspec(dllexport)
DWORDLONG fileDataBytes(fileHeader_t *f)
{
    return f->dataBytes; 
}

__declspec(dllexport)
void writeHeader(fileHeader_t *f)
{
    DWORD tmp;

    /* Write out the correct header info.       */
    SetFilePointer(f->fd, 0, NULL, FILE_BEGIN);

    if (f->compressed) {
        f->indexOffset = f->headerSize + f->dataBytes; 
    }
    else {
        f->dataBytes = (DWORDLONG)(f->readSize) * (DWORDLONG)(f->Nfields);
    }

    ZC (WriteFile(f->fd, f, sizeof(fileHeader_t), &tmp, NULL));
}

__declspec(dllexport)
void writeIndexBuffer(fileHeader_t *f, jFieldInfo_t *indexbuffer)
{
    int indexBytes;
    LONG hi_off, lo_off;
    DWORD tmp;

    indexBytes = (f->Nfields + 1) * sizeof(jFieldInfo_t);    

    hi_off = (LONG)(f->indexOffset>>32);
    lo_off = (LONG)(f->indexOffset & 0x00000000ffffffff);

    SetFilePointer(f->fd, lo_off, &hi_off, FILE_BEGIN);

    ZC (WriteFile(f->fd, indexbuffer, indexBytes, &tmp, NULL));
}

__declspec(dllexport)
void readIndexBuffer(fileHeader_t *f, jFieldInfo_t *indexbuffer)
{
    int indexBytes;
    LONG hi_off, lo_off;
    DWORD tmp;

    indexBytes = (f->Nfields + 1) * sizeof(jFieldInfo_t);

    hi_off = (LONG)(f->indexOffset>>32);
    lo_off = (LONG)(f->indexOffset & 0x00000000ffffffff);

    SetFilePointer(f->fd, lo_off, &hi_off, FILE_BEGIN);

    ZC (ReadFile(f->fd, indexbuffer, indexBytes, &tmp, NULL));
}

__declspec(dllexport)
jFieldInfo_t *createIndexBuffer(int nfields)
{
    int indexBytes;

    indexBytes = (nfields + 1) * sizeof(jFieldInfo_t);
    return (jFieldInfo_t *) malloc(indexBytes);
}
