/*
 * Copyright (c) 1998 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdio.h>
#include <stdlib.h>

#include <afxwin.h>

#include <gl/gl.h>
#include <gl/glu.h>

void
printExtensions(const char *printPrefix, const char *extensions,
                const char *extPrefix)
{
  char *p;
  char *extName;
  int prefixLen, lineLen;

  printf("%s extensions (%s):\n", printPrefix, extPrefix);
  if (extensions == NULL || extensions[0] == '\0')
    {
      puts ("    None.\n");
      return;
    }
  prefixLen = strlen(extPrefix);
  p = strdup(strstr(extensions, extPrefix));
  extName = strtok(p, " ");
  lineLen = printf("    %s", &extName[prefixLen]);
  while (extName = strtok(NULL, " "))
    {
      if (lineLen + strlen(extName) - prefixLen > 77)
	{
	  lineLen = printf(",\n    %s", &extName[prefixLen]) - 2;
        }
      else
	{
	  lineLen += printf(", %s", &extName[prefixLen]);
	}
    }
  puts(".");
  free( p );
}

void
gluinfo(void)
{
  printf("glu version: %s\n", gluGetString(GLU_VERSION));
  printExtensions("glu", (const char *)gluGetString(GLU_EXTENSIONS), "GLU_");
}

void main( int argc, char **argv )
{

  /* command line proc */
  if ( argc > 1 )
    {
    }
  
  /* create window, pfd, & opengl context */

  int index = 1;
  PIXELFORMATDESCRIPTOR pd;
  memset( &pd, 0, sizeof( pd ) );

  CFrameWnd *ww = new CFrameWnd;
  HDC dc = ::GetDC( ww->GetSafeHwnd() );

  SetPixelFormat( dc, 1, &pd );
  HGLRC rc = wglCreateContext( dc );

  /* query opengl implementation info */
	
  wglMakeCurrent( dc, rc );
  printf("OpenGL vendor string: %s\n", glGetString(GL_VENDOR));
  printf("OpenGL renderer string: %s\n", glGetString(GL_RENDERER));
  printf("OpenGL version string: %s\n", glGetString(GL_VERSION));

  /* query opengl utility info */
	
  //  gluinfo();
    
  /* query opengl extensions info */
	
  printExtensions("OpenGL", (char *) glGetString(GL_EXTENSIONS), "GL_");
    
  /* query visual info */
	
  printf( "\n" );

      printf( "---------------------------------------------------------------------------\n" );
      printf( "   visual hw dr dr    bf s@ ol ul rg d st  r  g  b a  ax dp st accum buffs\n" );
      printf( " id og pl ac wn bm gd sz pl pl pl ci b ro sz sz sz sz bf th cl  r  g  b  a\n" );
      printf( "---------------------------------------------------------------------------\n" );
    
  while( DescribePixelFormat( dc,  index,
			     sizeof(PIXELFORMATDESCRIPTOR), &pd ) != 0 )
    {
      printf( "%3d  %c  %c  %c  %c  %c  %c %2d  %c %2d %2d  %c %c  %c %2d %2d %2d %2d %2d %2d %2d ",
	     index,
	     ( pd.dwFlags & PFD_SUPPORT_OPENGL )?'y':'.',
	     ( pd.dwFlags & PFD_NEED_PALETTE )?'y':'.',
	     ( pd.dwFlags & PFD_GENERIC_FORMAT )?'n':'y',
	     ( pd.dwFlags & PFD_DRAW_TO_WINDOW )?'y':'n',
	     ( pd.dwFlags & PFD_DRAW_TO_BITMAP )?'y':'n',
	     ( pd.dwFlags & PFD_SUPPORT_GDI )?'y':'n',
	     pd.cColorBits,
	     ( pd.dwFlags & PFD_SWAP_LAYER_BUFFERS )?'y':'n',
	     ( ( pd.bReserved << 4 ) >> 4 ),
	     ( pd.bReserved >> 4 ),
	     ( pd.iPixelType == PFD_TYPE_RGBA )?'r':'c',
	     ( pd.dwFlags & PFD_DOUBLEBUFFER )?'y':'.',
	     ( pd.dwFlags & PFD_STEREO )?'y':'.',
	     pd.cRedBits,
	     pd.cGreenBits,
	     pd.cBlueBits,
	     pd.cAlphaBits,
	     pd.cAuxBuffers,
	     pd.cDepthBits,
	     pd.cStencilBits );
      printf( "%2d %2d %2d %2d",
	     pd.cAccumRedBits,
	     pd.cAccumGreenBits,
	     pd.cAccumBlueBits,
	     pd.cAccumAlphaBits
	     );
      printf( "\n" );
      memset( &pd, 0, sizeof( pd ) );
      index++;
    }
}
