/*
 * Copyright 1999 Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED
 *
 * You may use and modify this software and its documentation for your own
 * internal purposes without fee, provided that you agree that you will not
 * remove this notice from this software under any circumstances.  You may not
 * disclose the contents of this file to third parties in any form, in whole
 * or in part, without the prior written permission of Silicon Graphics, Inc.
 *
 * THIS SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT ANY WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
 * OF MERCHANTABLIITY OR FITNESS FOR A PARTICULAR PURPOSE.
 * In no event shall Silicon Graphics be liable for any special, incidental,
 * indirect or consequential damages of any kind, or any damages whatsoever
 * resulting from loss of use, data or profits, whether or not advised of the
 * possibility of damage, and on any theory of liability, arising out of or
 * in connection with the possession, use or performance of this software.
 *
 * U.S. GOVERNMENT RESTRICTED RIGHTS LEGEND
 * Use, duplication, or disclosure by the Government is subject to restrictions
 * set forth in FAR 52.227.19(c)(2) or subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013 and/or in
 * similar or successor clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the United States.
 * Contractor/manufacturer is Silicon Graphics, Inc., 2011 N. Shoreline Blvd.,
 * Mountain View, CA 94039-7311.
 *
 * Author: Jacques Lefaucheux   jacques@paris.sgi.com
 * January 20, 1999
 */

// include files
// windows.h
// must be included before GL/gl.h

#include <windows.h>
#include <GL/gl.h>		// OpenGL header file
#include <GL/glu.h>		// OpenGL utilities
#include <stdio.h>		// io

#include "overlays.h"

// pragmas
#pragma warning(disable : 4244)		// disable conversion warnings

#define WIDTH  500
#define HEIGHT 500

// rubberband variables
int rubb_color=1;
LONG start_x,start_y,delta_x,delta_y,end_x,end_y;
// globals 
HDC   hDC;				// device context 
HGLRC hRC;				// opengl main plane context
HGLRC hOvlRC;			// opengl overlay context 
HWND  hWnd;			    // window

LONG mx,my; // mouse position
LONG wheight=0,wwidth=0;

void fin(){
    /* clean up */
    wglMakeCurrent(NULL, NULL);	// make our context 'un-'current 
    ReleaseDC(hDC, hWnd);		// release handle to DC 
    wglDeleteContext(hRC);		// delete the rendering context 
    wglDeleteContext(hOvlRC);	//delete the overlay context 
    DestroyWindow(hWnd);		// destroy the window 
	exit(0);
}

void ValidatePaint(){
PAINTSTRUCT ps;
	wglMakeCurrent(hDC, NULL);
	BeginPaint(hWnd, &ps); 
	EndPaint(hWnd, &ps); 
}

void setupViewport(int w,int h){
	wglMakeCurrent(hDC, hRC);
	glViewport(0, 0, w, h);
	wglMakeCurrent(hDC, hOvlRC);	
	glViewport(0, 0, w, h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0.0,(double)w, 0.0,(double)h);
	glMatrixMode(GL_MODELVIEW);
	wheight = h;
	wwidth  = w;
}

void Draw() // draw triangle
{
	wglMakeCurrent(hDC, hRC);
	glClearColor(0.5,0.5,0.5,0.0); // blue background
	glClear(GL_COLOR_BUFFER_BIT);  // clear
	glRotatef(1.0, 0.0, 0.0, 1.0); // rotate triangles
	glBegin(GL_TRIANGLES);
	glColor3f(1.0, 0.0, 0.0);
	glVertex2i( 0,  1);
	glColor3f(0.0, 1.0, 0.0);
	glVertex2i(-1, -1);
	glColor3f(0.0, 0.0, 1.0);
	glVertex2i( 1, -1);
	glEnd();

	wglSwapLayerBuffers(hDC, WGL_SWAP_MAIN_PLANE); // swap buffers
	ValidatePaint();
}

void DrawOvl() // draw rubberband
{
	wglMakeCurrent(hDC, hOvlRC);
	glClearIndex(0);
	glClear(GL_COLOR_BUFFER_BIT);
	glBegin(GL_LINE_LOOP);	
	glIndexi(rubb_color);
	glVertex2i(start_x,start_y);
	glVertex2i(start_x+delta_x,start_y);
	glVertex2i(start_x+delta_x,start_y+delta_y);
	glVertex2i(start_x,start_y+delta_y);
	glEnd();
	if (OVL_SINGLE)
		glFlush();
	else
		wglSwapLayerBuffers(hDC,WGL_SWAP_OVERLAY1);
	ValidatePaint();

}

void changeRubbColor()
{
rubb_color++;
if ( rubb_color== MAX_OVERLAY_COLORS) rubb_color =1;
}

LONG WINAPI WindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{ 
    LONG        lRet = 0;
	RECT rect;
	static char start_rubb = 0;
	static char first = 1;
	

#ifdef DEBUG_MSG
	printf("I am in Windows proc: message <0x%4x> %d %l\n",uMsg,wParam,lParam);
#endif
	switch(uMsg) {

    case WM_CREATE:
		break; 
    case WM_DESTROY: 
		break; 
	case WM_ERASEBKGND:
		break;	
    case WM_PAINT:
		if (first){
			setupViewport(wwidth,wheight);
			first = 0;
		}
		Draw(); // Draw Main Plane
		break;

	case WM_LBUTTONDOWN:
//		printf("\t WM_LBUTTONDOWN\n");
		start_x = mx;
		start_y = my;
		start_rubb = 1;

		// Change rubberband color
		changeRubbColor();
		break;

	case WM_LBUTTONUP:
//		printf("\t WM_LBUTTONUP\n");
		start_rubb = 0;
		end_x = mx;
		end_y = my;
		break;

	case WM_MOUSEMOVE:
		// draw overlays
		if (start_rubb == 0) break;
		delta_x = mx-start_x;
		delta_y = my-start_y;
//		printf("\t WM_MOUSEMOVE %d %d %d %d %d %d\n",mx,my,start_x,start_y,delta_x,delta_y);
		
		DrawOvl(); // DrawOverlays
        break; 

    case WM_CHAR:
		if(wParam == 27)		/* ESC */
			PostQuitMessage(0);
			break;

    case WM_SIZE:
//		printf("\t WM_SIZE %d %d \n",LOWORD(lParam), HIWORD(lParam));
		setupViewport((int)LOWORD(lParam), (int)HIWORD(lParam));
		break;

	case WM_WINDOWPOSCHANGED:
//		printf("\t WM_WINDOWPOSCHANGED \n");
		GetClientRect(hWnd,&rect);
//		printf("LRTB = %d %d %d %d \n",rect.left,rect.right,rect.top,rect.bottom);
		setupViewport(rect.right,rect.bottom);
		SendMessage(hWnd,WM_PAINT,0,0);
		break;

    case WM_CLOSE:
	     PostQuitMessage(0);
		break;

	case WM_QUIT:
		fin();
		break;

    default: 
        lRet = DefWindowProc(hWnd, uMsg, wParam, lParam); 
        break; 
    }

    return lRet; 
} 



// Create a window suitable for OpenGL rendering

HWND oglCreateWindow(char* title, int x, int y, int width, int height){
    WNDCLASS  wc;
    HWND      hWnd;
    HINSTANCE hInstance;

    /* get this modules instance */
    hInstance = GetModuleHandle(NULL);

    /* fill in the window class structure */
    wc.style         = CS_OWNDC ;                   /* no special styles */
    wc.lpfnWndProc   = (WNDPROC)WindowProc;         /* event handler */
    wc.cbClsExtra    = 0;                           /* no extra class data */
    wc.cbWndExtra    = 0;                           /* no extra window data */
    wc.hInstance     = hInstance;                   /* instance */
    wc.hIcon         = LoadIcon(NULL, IDI_WINLOGO); /* load a default icon */
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW); /* load a default cursor */
    wc.hbrBackground = NULL;                        /* redraw our own bg */
    wc.lpszMenuName  = NULL;                        /* no menu */
    wc.lpszClassName = title;                       /* use a special class */

    /* register the window class */
    if (!RegisterClass(&wc)) {
      MessageBox(NULL, 
		   "RegisterClass() failed:  Cannot register window class,",
		   "Error", MB_OK);
	return NULL;
    }

    /* create a window */
    hWnd = CreateWindow(title,          /* class */
			title,						/* title */
			WS_OVERLAPPEDWINDOW |		/* window attributes */
			WS_CLIPSIBLINGS | WS_CLIPCHILDREN,			/* style */
			x, y, width, height,		/* dimensions */
			NULL,						/* no parent */
			NULL,						/* no menu */
			hInstance,					/* instance */
			NULL);						/* don't pass anything to WM_CREATE */

    /* make sure we got a window */
    if (hWnd == NULL) {
	MessageBox(NULL,
		   "CreateWindow() failed:  Cannot create a window.",
		   "Error", MB_OK);
	return NULL;
    }
    return hWnd;
}


void compute_mouse_position(HWND hWnd,POINT mouse,int *mx,int *my){
	ScreenToClient(hWnd,&mouse); 
	*mx = mouse.x; // store mouse position
	*my = wheight-mouse.y; 
}

int main(int argc, char** argv){
    MSG       msg;			/* message */
    int first = 1;

	// create a window
    if ( (hWnd = oglCreateWindow("OpenGL Overlay test", 0, 0, WIDTH,HEIGHT) ) == NULL)
        exit(1);

    // Get Device context
    hDC = GetDC(hWnd);

    // Create OpenGL context with Overlay and Main Plane Pixel format
   if (createContextAndOverlays(hDC,&hRC,&hOvlRC,0) == 0){
		printf("Error setting Overlays colors \n");
		exit(1);
	}

    // show the window (map it)
    ShowWindow(hWnd, SW_SHOW);
	UpdateWindow(hWnd);

    // event management 
    while(GetMessage(&msg, hWnd, 0, 0) > 0){
		compute_mouse_position(hWnd,msg.pt,&mx,&my);
		TranslateMessage(&msg); // change virtual key to message
		DispatchMessage(&msg);  // call WinProc	
	}
return 1;
}
