/*
 * File:           wglinfo.c
 * Author:         Nate Robins
 * Created:        Jan 20, 1997
 * Description:

 *   WGL/OpenGL visual information program
 *
 * Copyright (c) 1997, Paramtric Technology Corporation, all rights reserved.
 */


// includes
#include <windows.h>
#include <stdio.h>
#include <GL/gl.h>

// pragmas
#pragma warning(disable : 4244)  // disable conversion warnings on intel


// functions

/* VisualInfo()
 *  Shows a graph of all the visuals that support OpenGL
 *  and their capabilities.  Just like glxinfo on SGI's.
 */
void VisualInfo(HDC hDC)
{
    int                   i, maxpf;
    char*                 s;
    char                  t[80];
    char*                 p;
    PIXELFORMATDESCRIPTOR pfd;

    maxpf = DescribePixelFormat(hDC, 0, 0, NULL);

    printf("hDC: 0x%x\n", hDC);
    printf("server wgl vendor string: N/A\n");
    printf("server wgl version string: N/A\n");
    printf("server wgl extensions (WGL_): N/A\n");
    printf("client wgl version: N/A\n");
    printf("client wgl extensions (WGL_): none\n");
    printf("OpenGL vendor string: %s\n", glGetString(GL_VENDOR));
    printf("OpenGL renderer string: %s\n", glGetString(GL_RENDERER));
    printf("OpenGL version string: %s\n", glGetString(GL_VERSION));
    printf("OpenGL extensions (GL_): \n");
    i = 0;
    s = (char*)glGetString(GL_EXTENSIONS);
    t[79] = '\0';
    while(*s) {
	t[i++] = *s;
	if(*s == ' ') {
	    t[i-1] = ',';
	    t[i] = ' ';
	    p = &t[i++];
	}
	if(i > 80 - 5) {
	    *p = t[i] = '\0';
	    printf("    %s\n", t);
	    p++;
	    i = strlen(p);
	    strcpy(t, p);
	}
	s++;
    }
    t[i] = '\0';
    printf("    %s.\n\n", t);

    printf("   visual  x  bf lv rg d st  r  g  b a  ax dp st accum buffs  ms \n");
    printf(" id dep cl sp sz l  ci b ro sz sz sz sz bf th cl  r  g  b  a ns b\n");
    printf("-----------------------------------------------------------------\n");
    for(i = 1; i <= maxpf; i++) {
	DescribePixelFormat(hDC, i, sizeof(PIXELFORMATDESCRIPTOR), &pfd);
	if(!(pfd.dwFlags & PFD_SUPPORT_OPENGL))
	    continue;  // skip non-opengl visuals
	printf("0x%02x ", i);
	printf("%2d ", pfd.cColorBits);
	if(pfd.dwFlags & PFD_DRAW_TO_WINDOW)      printf("wn ");
	else if(pfd.dwFlags & PFD_DRAW_TO_BITMAP) printf("bm ");
	else printf(".  ");
	printf(" . "); // x sp = transparent ???
	printf("%2d ", pfd.cColorBits);
//	if(pfd.iLayerType) printf(" %d ", pfd.iLayerType);  // bReserved --v
	if(pfd.bReserved) printf(" %d ", pfd.bReserved);
	else printf(" . "); 
	printf(" %c ", pfd.iPixelType == PFD_TYPE_RGBA ? 'r' : 'c');
	printf("%c ", pfd.dwFlags & PFD_DOUBLEBUFFER ? 'y' : '.');
	printf(" %c ", pfd.dwFlags & PFD_STEREO ? 'y' : '.');
	if(pfd.cRedBits)        printf("%2d ", pfd.cRedBits);
	else printf(" . ");
	if(pfd.cGreenBits)      printf("%2d ", pfd.cGreenBits);
	else printf(" . ");
	if(pfd.cBlueBits)       printf("%2d ", pfd.cBlueBits);
	else printf(" . ");
	if(pfd.cAlphaBits)      printf("%2d ", pfd.cAlphaBits);
	else printf(" . ");
	if(pfd.cAuxBuffers)     printf("%2d ", pfd.cAuxBuffers);
	else printf(" . ");
	if(pfd.cDepthBits)      printf("%2d ", pfd.cDepthBits);
	else printf(" . ");
	if(pfd.cStencilBits)    printf("%2d ", pfd.cStencilBits);
	else printf(" . ");
	if(pfd.cAccumRedBits)   printf("%2d ", pfd.cAccumRedBits);
	else printf(" . ");
	if(pfd.cAccumGreenBits) printf("%2d ", pfd.cAccumGreenBits);
	else printf(" . ");
	if(pfd.cAccumBlueBits)  printf("%2d ", pfd.cAccumBlueBits);
	else printf(" . ");
	if(pfd.cAccumAlphaBits) printf("%2d ", pfd.cAccumAlphaBits);
	else printf(" . ");
	printf(" . .\n");  // multisample stuff
    }

    printf("-----------------------------------------------------------------\n");
    printf("   visual  x  bf lv rg d st  r  g  b a  ax dp st accum buffs  ms \n");
    printf(" id dep cl sp sz l  ci b ro sz sz sz sz bf th cl  r  g  b  a ns b\n");
    printf("-----------------------------------------------------------------\n");

}

/* WindowProc()
 *  Minimum Window Procedure
 */
LONG WINAPI WindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{ 
    LONG        lRet = 1;
    PAINTSTRUCT ps;

    switch(uMsg) {
    case WM_CREATE:
        break; 

    case WM_DESTROY:
        break; 

    case WM_PAINT: 
        BeginPaint(hWnd, &ps); 
        EndPaint(hWnd, &ps); 
        break; 

    default: 
        lRet = DefWindowProc (hWnd, uMsg, wParam, lParam); 
        break; 
    }

    return lRet; 
} 

/* main()
 *  Entry point
 */
int main(int argc, char** argv)
{
    int       pf;
    HDC       hDC;
    HGLRC     hRC;
    HWND      hWnd;
    WNDCLASS  wc;
    HINSTANCE hInstance;
    PIXELFORMATDESCRIPTOR pfd;
    char*     WindowClass = "OpenGL";

    // get this modules instance
    hInstance = GetModuleHandle(NULL);

    // fill in the window class structure
    wc.style         = 0;                            // no special styles
    wc.lpfnWndProc   = (WNDPROC)WindowProc;          // event handler
    wc.cbClsExtra    = 0;                            // no extra class data
    wc.cbWndExtra    = 0;                            // no extra window data
    wc.hInstance     = hInstance;                    // instance
    wc.hIcon         = LoadIcon(NULL, IDI_WINLOGO);  // load a default icon
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);  // load a default cursor
    wc.hbrBackground = NULL;                         // redraw our own bg
    wc.lpszMenuName  = NULL;                         // no menu
    wc.lpszClassName = WindowClass;                  // use a special class

    // register the window class
    if(!RegisterClass(&wc)) {
	MessageBox(NULL, 
		   "RegisterClass() failed:  Cannot register window class,",
		   "Error", MB_OK);
	return(0);
    }

    // create a window
    hWnd = CreateWindow(WindowClass,     // class
			WindowClass,     // name   
			WS_CLIPSIBLINGS | WS_CLIPCHILDREN,  // style
			0, 0, 200, 200,  // x, y, width, height
			NULL,            // no parent
			NULL,            // no menu
			hInstance,       // instance
			NULL);           // don't pass anything to WM_CREATE

    // make sure we got a window
    if(hWnd == NULL) {
	MessageBox(NULL,
		   "CreateWindow() failed:  Cannot create a window.",
		   "Error", MB_OK);
	return(0);
    }

#if 0
    // show the window (map it)
    ShowWindow(hWnd, SW_SHOW);

    // send an initial WM_PAINT message (expose)
    UpdateWindow(hWnd);
#endif

    // grab the device context
    hDC = GetDC(hWnd);
    
    // fill in the pixel format descriptor
    pfd.nSize        = sizeof(PIXELFORMATDESCRIPTOR);
    pfd.nVersion     = 1;
    pfd.dwFlags      = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL;
    pfd.dwLayerMask  = PFD_MAIN_PLANE;
    pfd.iPixelType   = PFD_TYPE_RGBA;
    pfd.cColorBits   = 24;
    pfd.cDepthBits   = 16;
    pfd.cAccumBits   = 0;
    pfd.cStencilBits = 0;
 
    // get the appropriate pixel format
    if((pf = ChoosePixelFormat(hDC, &pfd)) == 0) {
       MessageBox(NULL,
		  "ChoosePixelFormat() failed:  Cannot find format specified.",
		  "Error", MB_OK); 
       return(0);
    } 
 
    // set the pixel format
    if(SetPixelFormat(hDC, pf, &pfd) == FALSE) {
	MessageBox(NULL,
		   "SetPixelFormat() failed:  Cannot set format specified.",
		   "Error", MB_OK);
        return(0);
    } 
    
    // create a context
    hRC = wglCreateContext(hDC);
    wglMakeCurrent(hDC, hRC);

    // show all visuals
    VisualInfo(hDC);

#if 0
    int i;

    for(i = 16; i > 0; i--) {
	glClearColor(i/16.0, 0.0, 0.0, 0.0);
	glClear(GL_COLOR_BUFFER_BIT);
	Sleep(500);
    }
#endif

    // clean up
    wglDeleteContext(hRC);
    DestroyWindow(hWnd);
}
