/*
 * Copyright (c) 1998-99 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdlib.h>

int m104597[256];
int m76309[256];
int m132201[256];
int m25675[256];
int m53279[256];
int clamp[256];

#define M104597(x) (104597*((x)-128))
#define M76309(x) (76309*((x)-16))
#define M132201(x) (132201*((x)-128))
#define M25675(x) (25675*((x)-128))
#define M53279(x) (53279*((x)-128))

__declspec(dllexport)
void init_422_tables()
{
    int i;
    for(i=0;i<255;i++){
        m104597[i]=M104597(i);
        m76309[i]=M76309(i) + 32772;
        m132201[i]=M132201(i);
        m25675[i]=M25675(i);
        m53279[i]=M53279(i);

        if (i < 128) clamp[i] = 0xff000000;
        else clamp[i] = 0;
    }
}

#undef M104597
#undef M76309
#undef M132201
#undef M25675
#undef M53279

#define M104597(x) m104597[x]
#define M76309(x)  m76309[x]
#define M132201(x) m132201[x]
#define M25675(x)  m25675[x]
#define M53279(x)  m53279[x]

#define GET_YUV422 \
        u0 = *uyvy++ ; \
        y0 = *uyvy++ ; \
        v0 = *uyvy++ ; \
        y1 = *uyvy++ 

#define RGB_CNV(yy, uu, vv) \
        y_m = M76309(yy); \
        red = M104597(vv) + y_m; \
        blue = M132201(uu) + y_m; \
        green = y_m - M25675(uu) - M53279(vv); \
	    if (red >> 24) red = ~(red >> 24); \
	    if (blue >> 24) blue = ~(blue >> 24); \
	    if (green >> 24) green = ~(green >> 24)

__declspec(dllexport)
void fast422ToRGB(int width, int height, int *buf1, int *buf2, int interlace)
{
	int frame_size;
	int frame_size_int;
	int u0,y0,v0,y1, y_m,red,blue,green;
	int i;
	int k;
    int linecount = 0;
	unsigned char *uyvy;

    frame_size = width * height * 2;
	frame_size_int=frame_size/sizeof(int);
	uyvy=(unsigned char *)buf1;


    for(k=0;k<frame_size_int-1;){

            /* Do first two pixels	*/
		    GET_YUV422;

		   /*
		    * First pixel is (y0, u0, v0)
		    */
		    RGB_CNV(y0,u0,v0);
		    red   = ((red & 0xff0000)   >> 16);
		    green = ((green & 0xff0000) >>  8);
		    blue  = ((blue & 0xff0000)       );
		    buf2[2*k] = red | green | blue;

		   /*
		    * Second pixel is (y1, u0, v0)
		    */
		    RGB_CNV(y1,u0, v0);
		    red   = ((red & 0xff0000)   >> 16);
		    green = ((green & 0xff0000) >>  8);
		    blue  = ((blue & 0xff0000)       );
		    buf2[2*k+1] = red | green | blue;

            k++;

            /* Do second two pixels	*/
		    GET_YUV422;

		   /*
		    * First pixel is (y0, u0, v0)
		    */
		    RGB_CNV(y0,u0,v0);
		    red   = ((red & 0xff0000)   >> 16);
		    green = ((green & 0xff0000) >>  8);
		    blue  = ((blue & 0xff0000)       );
		    buf2[2*k] = red | green | blue;

		   /*
		    * Second pixel is (y1, u0, v0)
		    */
		    RGB_CNV(y1,u0, v0);
		    red   = ((red & 0xff0000)   >> 16);
		    green = ((green & 0xff0000) >>  8);
		    blue  = ((blue & 0xff0000)       );
		    buf2[2*k+1] = red | green | blue;

            k++;

            linecount += 4;

            if ((interlace) && (linecount == width)) {
                linecount = 0;
                buf2 += width;
            }
    }

}

/* Similar to above routine but this one averages the u's and v's
 * for the second pixel of each pair.
 */

__declspec(dllexport)
void fast422ToRGB_Filter(int width, int height, int *buf1, int *buf2, int interlace)
{
	int frame_size;
	int frame_size_int;
	int u0,y0,v0,y1, y_m,red,blue,green;
  	int lu0, ly0, lv0, ly1;
	int i;
	int j;
	int k;
	unsigned char *uyvy;
    int linecount = 0;

    frame_size = width * height * 2;
	frame_size_int=frame_size/sizeof(int);
	uyvy=(unsigned char *)buf1;

    GET_YUV422;
    for(k=0;k<frame_size_int-1;){
		    lu0 = u0;
		    ly0 = y0;
		    lv0 = v0;
		    ly1 = y1;
		    GET_YUV422;

           /* Do first two pixels.	*/ 

		   /*
		    * First pixel is (y0, u0, v0)
		    */
		    RGB_CNV(ly0,lu0,lv0);
		    red   = ((red & 0xff0000)   >> 16);
		    green = ((green & 0xff0000) >>  8);
		    blue  = ((blue & 0xff0000)       );
		    buf2[2*k] = red | green | blue;

		   /*
		    * Second pixel is (y1, (u0+u2)/2, (v0+v2)/2 )
		    * i.e. average chroma from this uyvy and the
		    * next.
		    */
		    RGB_CNV(ly1,(lu0+u0)>>1,(lv0+v0)>>1);
		    red   = ((red & 0xff0000)   >> 16);
		    green = ((green & 0xff0000) >>  8);
		    blue  = ((blue & 0xff0000)       );
		    buf2[2*k+1] = red | green | blue;

		    k++;

           /* Do second two pixels.	*/ 

		    lu0 = u0;
		    ly0 = y0;
		    lv0 = v0;
		    ly1 = y1;
		    GET_YUV422;

		   /*
		    * First pixel is (y0, u0, v0)
		    */
		    RGB_CNV(ly0,lu0,lv0);
		    red   = ((red & 0xff0000)   >> 16);
		    green = ((green & 0xff0000) >>  8);
		    blue  = ((blue & 0xff0000)       );
		    buf2[2*k] = red | green | blue;

		   /*
		    * Second pixel is (y1, (u0+u2)/2, (v0+v2)/2 )
		    * i.e. average chroma from this uyvy and the
		    * next.
		    */
		    RGB_CNV(ly1,(lu0+u0)>>1,(lv0+v0)>>1);
		    red   = ((red & 0xff0000)   >> 16);
		    green = ((green & 0xff0000) >>  8);
		    blue  = ((blue & 0xff0000)       );
		    buf2[2*k+1] = red | green | blue;

		    k++;

            linecount += 4;

            if ((interlace) && (linecount == width)) {
                linecount = 0;
                buf2 += width;
            }
		}

		/*
		 * For last two pixels, reflect chroma on right edge
		 */
		RGB_CNV(y0,u0,v0);
        red   = ((red & 0xff0000)   >> 16);
        green = ((green & 0xff0000) >>  8);
        blue  = ((blue & 0xff0000)       );
		buf2[2*k] = red | green | blue;
		RGB_CNV(y1,(lu0+u0)>>1,(lv0+v0)>>1);
        red   = ((red & 0xff0000)   >> 16);
        green = ((green & 0xff0000) >>  8);
        blue  = ((blue & 0xff0000)       );
		buf2[2*k+1] = red | green | blue;
}
