/*
 * Copyright (c) 1997, 1999 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef handy_h
#define handy_h

/*
 * Handy error macros
 */

#ifdef __ANSI_CPP__
#define PRINT_CALL(call) \
   fprintf( stderr, "%s failed in file %s, line %d\n", # call, __FILE__, __LINE__ );
#else
#define PRINT_CALL(call) \
   fprintf( stderr, "%s failed in file %s, line %d\n", "call", __FILE__, __LINE__ );
#endif

#define ERR(x, message)							                            \
    {										                                \
        if ( (x) ) {							                            \
            fprintf( stderr, "%s\n", message );					            \
            fprintf( stderr, "in file %s, line %d\n", __FILE__, __LINE__);	\
            while (1) Sleep(10);                                            \
	    }								                                    \
    }									                                    \

/*
 ** LTC - flags return values less than 0. 
 */

#define LTC(call)				  \
    {						      \
	    if ( (call) < 0 ) {	      \
	        PRINT_CALL(call);     \
            while(1) Sleep(10);   \
	    }						  \
    }						      \

/*
 ** NC - flags return values that are NULL 
 */

#define NC(call)			      \
    {						      \
	    if ( (call) == NULL ) {   \
	        PRINT_CALL(call);	  \
            while(1) Sleep(10);   \
	    }				          \
    }						      \

/*
 ** ZC - flags return values that are 0 
 */

#define ZC(call)			      \
    {						      \
	    if ( (call) == 0 ) {      \
	        PRINT_CALL(call);	  \
            while(1) Sleep(10);   \
	    }				          \
    }						      \

#endif
