#include "trackball.h"

#include "stdafx.h"

#include <math.h>

#include <Cosmo3D/csBasic.h>
#include <Cosmo3D/csTransform.h>

inline float square( float xx ) { return( xx*xx ); }

trackball::trackball( csTransform *spin, csTransform *ref )
{
  _spin = spin;
  _ref = ref;
}

trackball::~trackball() {}

bool trackball::updateTrackball( int xx, int yy, 
				 int dx, int dy,
				 int width, int height )
{
  csRotation rot;
  _spin->getRotation( rot );

  yy = height-yy;
  dx *= -1.0;

   csVec3f    p0, p1, v0, v1;
   csVec3f    sv;
   double     mag;
   long       Xcenter, Ycenter;

   // First, compute the incremental rotation from the mouse positions.

   // Get screen size to normalize x/y cursor motion to unity.
   Xcenter = width  / 2;
   Ycenter = height / 2;

   // Compute the old and new cursor position on
   // the virtual sphere
   p0[0] = (xx - (float)Xcenter) * 2 / width;
   p0[1] = (yy - (float)Ycenter) * 2 / height;
   p0[2] = 1 - square(p0[0]) - square(p0[1]);

   if ( p0[2] < 0 )
   {
      p0[2] = 0;
   }
   else
   {
      p0[2] = sqrtf( p0[2] );
   }

   p1[0] = (xx + dx - (float)Xcenter) * 2 / width;
   p1[1] = (yy + dy - (float)Ycenter) * 2 / height;
   p1[2] = 1 - square(p1[0]) - square(p1[1]);

   if ( p1[2] < 0 )
   {
      p1[2] = 0;
   }
   else
   {
      p1[2] = sqrtf( p1[2] );
   }

   // Let vectors v0 and v1 represent the
   // plane of rotation defined by p0 and p1
   v0 = p0;
   v1 = p1;
   v0.normalize();
   v1.normalize();

   // Let sv be the normal to that plane
   sv.cross( v0, v1 );
   mag = sv.length();

   // Now, accumulate the incremental rotation in the old rotation.
   if ( mag != 0 )
   {
      // Create the new incremental rotation.
      csRotation newRot( sv[0], sv[1], sv[2], -asin(mag)*2.0 );

      // Express the axis of the incremental rotation wrt the cordinate system
      // described by "_ref".  To do this we compute the dot product of
      // each row of a matrix M with the axis of "rot". Where M is the 3X3 upper
      // left matrix of "_ref" (i.e. the rotation portion of the matrix).
      csMatrix4f matr;
      _ref->getMatrix( matr );
      matr.transpose( matr );
      csVec3f axis;
      newRot.getAxis( axis );
      axis.xformVec( axis, matr );

      // Set the rotation axis to the new axis.
      newRot.setAxis( axis );

      // Accumulate the new rotation in "rot".
      rot.mult( rot, newRot );

      // Apply it back to the transform

      _spin->setRotation( rot );

      // Tell the caller whether something really changed
      return true;
   }
   else
   {
      return false;
   }
}
