#include "stdafx.h"
#include "app.h"
#include "gfxmgr.h"
#include "doc.h"
#include "view.h"

#include <windows.h>
#include <winbase.h>

#include <gl/gl.h>
#include <gl/glu.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// oglView

IMPLEMENT_DYNCREATE(oglView, CView)

BEGIN_MESSAGE_MAP(oglView, CView)
	//{{AFX_MSG_MAP(oglView)
	ON_WM_ERASEBKGND()
	ON_WM_SIZE()
	ON_WM_CREATE()
	ON_WM_MOUSEMOVE()
        ON_WM_MBUTTONDBLCLK()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// oglView construction/destruction

oglView::oglView()
{
	// TODO: add construction code here
}

oglView::~oglView()
{
}

BOOL oglView::PreCreateWindow(CREATESTRUCT& cs)
{
  //the view windows style bits must include WS_CLIPSIBLINGS and
  //WS_CLIPCHILDREN so that the wgl functions will work. This is
  
  cs.style = cs.style | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
  
  return( CView::PreCreateWindow(cs) );
}

/////////////////////////////////////////////////////////////////////////////
// oglView drawing

void oglView::OnDraw(CDC* pDC)
{
  oglDoc* pDoc = GetDocument();
  ASSERT_VALID(pDoc); 

  /// User code follows ///
  
  if ( wglGetCurrentContext() != _gl_context )
  {
    wglMakeCurrent( _dev_context, _gl_context );
  }
  
#ifdef _LITTLE_TEST_TO_SEE_IF_DRAWING_IS_HAPPENING_
    {
      char msg[ 256 ];
      sprintf( msg, "Punching Draw!", GetLastError() );
      MessageBox( msg, NULL, MB_ICONINFORMATION | MB_OK );
    }
#endif

    oglmgr->clear();
    oglmgr->draw();  

  SwapBuffers( _dev_context );
}

/////////////////////////////////////////////////////////////////////////////
// oglView diagnostics

#ifdef _DEBUG
void oglView::AssertValid() const
{
	CView::AssertValid();
}

void oglView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

oglDoc* oglView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS( oglDoc )));
	return (oglDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// oglView message handlers

BOOL oglView::OnEraseBkgnd(CDC* pDC) 
{
	// overriding this prevents an extraneous white clear by the window
	// prior to redrawing the ogl goop.
	return( TRUE );
}

void oglView::OnSize(UINT nType, int ww, int hh ) 
{
	CView::OnSize( nType, ww, hh );
	oglmgr->resize( ww, hh );
	static int firsttime = 1;
#if 0
	if ( firsttime )
	  {
	    firsttime = 0;
	  }
	else
	  {
	    char msg[ 256 ];
	    sprintf( msg, "resize to %d %d\n", ww, hh );
	    MessageBox( msg, NULL, MB_ICONINFORMATION | MB_OK );
	  }
#endif
}

int oglView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
  if (CView::OnCreate(lpCreateStruct) == -1)
    return( -1 );

    PIXELFORMATDESCRIPTOR pfd = {
	sizeof(PIXELFORMATDESCRIPTOR),	/* size of this pfd */
	1,				/* version num */
	PFD_DRAW_TO_WINDOW |		/* support window */
	PFD_SUPPORT_OPENGL |		/* support OpenGL */
	PFD_DOUBLEBUFFER,		/* support double buffering */
	PFD_TYPE_RGBA,			/* RGBA buffer */
	8,				/* 8-bit color depth */
	0, 0, 0, 0, 0, 0,		/* color bits (ignored) */
	0,				/* no alpha buffer */
	0,				/* alpha bits (ignored) */
	0,				/* no accumulation buffer */
	0, 0, 0, 0,			/* accum bits (ignored) */
	16,				/* 16-bit depth buffer */
	0,				/* no stencil buffer */
	0,				/* no auxiliary buffers */
	PFD_MAIN_PLANE,			/* main layer */
	0,				/* reserved */
	0, 0, 0				/* no layer, visible, damage masks */
    };
    int SelectedPixelFormat;
    BOOL retVal;

    _dev_context =  ::GetDC( GetSafeHwnd() );
    SelectedPixelFormat = ChoosePixelFormat( _dev_context, &pfd);
    if (SelectedPixelFormat == 0)
    {
      char msg[ 256 ];
      sprintf( msg, "Failed to find acceptable pixel format. Error = %d\n",
	       GetLastError());
      MessageBox( msg, NULL, MB_ICONSTOP | MB_OK );
      exit(1);
    }

    retVal = SetPixelFormat( _dev_context, SelectedPixelFormat, &pfd);
    if (retVal != TRUE)
      {
	char msg[ 256 ];
	sprintf( msg, "Failed to set acceptable pixel format. Error = %d\n",
	    GetLastError());
	MessageBox( msg, NULL, MB_ICONSTOP | MB_OK );
	exit(1);
    }
    
#ifdef _DEBUG_PRINT_PIXELFORMAT
    {
      char msg[ 256 ];
      sprintf( msg, "Using PixelFormat: %d\n", GetPixelFormat( _dev_context ) );
      MessageBox( msg, NULL, MB_ICONSTOP | MB_OK );
    }
#endif

    setupPalette( _dev_context );

    _gl_context = wglCreateContext( _dev_context );
    if( _gl_context == NULL )
      {
	char msg[ 256 ];
	sprintf( msg, "GL context creation failed: %d\n", GetLastError() );
	MessageBox( msg, NULL, MB_ICONSTOP | MB_OK );
	exit( -1 );
      }

    if ( TRUE == wglMakeCurrent( _dev_context, _gl_context ) )
      {
	RECT rc;
	GetClientRect( &rc );

	oglmgr = new gfxmgr;
	oglmgr->init( rc.right, rc.bottom );
      }

    return( FALSE );
}

HPALETTE
oglView::setupPalette(HDC hDC)
{
    PIXELFORMATDESCRIPTOR pfd;
    LOGPALETTE* pPal;
    HPALETTE viewerPalette;
    int pixelFormat = GetPixelFormat(hDC);
    int paletteSize;

    (void) DescribePixelFormat(hDC, pixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &pfd);
    if (!(pfd.dwFlags & PFD_NEED_PALETTE || pfd.iPixelType == PFD_TYPE_COLORINDEX)) {
	return NULL;
    }

    paletteSize = 1 << pfd.cColorBits;
    pPal = (LOGPALETTE*)
	malloc(sizeof(LOGPALETTE) + paletteSize * sizeof(PALETTEENTRY));
    pPal->palVersion = 0x300;
    pPal->palNumEntries = paletteSize;

    /* start with a copy of the current system palette */
    (void) GetSystemPaletteEntries(hDC, 0, paletteSize, &pPal->palPalEntry[0]);

    {
	/* fill in an RGBA color palette */
	int redMask = (1 << pfd.cRedBits) - 1;
	int greenMask = (1 << pfd.cGreenBits) - 1;
	int blueMask = (1 << pfd.cBlueBits) - 1;
	int i;

	for (i=10; i<paletteSize-10; ++i) {
	    pPal->palPalEntry[i].peRed =
		    (((i >> pfd.cRedShift) & redMask) * 255) / redMask;
	    pPal->palPalEntry[i].peGreen =
		    (((i >> pfd.cGreenShift) & greenMask) * 255) / greenMask;
	    pPal->palPalEntry[i].peBlue =
		    (((i >> pfd.cBlueShift) & blueMask) * 255) / blueMask;
	    pPal->palPalEntry[i].peFlags = PC_NOCOLLAPSE;
	}
    }

    viewerPalette = CreatePalette(pPal);
    free(pPal);

    if (viewerPalette) {
	(void) SelectPalette(hDC, viewerPalette, FALSE);
	(void) RealizePalette(hDC);
    }

    return( viewerPalette );
}

void oglView::OnMouseMove(UINT nFlags, CPoint point) 
{
  if ( nFlags & MK_LBUTTON )
    {
      float pos = oglmgr->getPosition();
      pos += .1;
      if ( pos > 50 ) pos = 50;
      oglmgr->setPosition( pos );
      Invalidate( FALSE );
    }
  else if ( nFlags & MK_RBUTTON )
    {
      float pos = oglmgr->getPosition();
      pos -= .1;
      if ( pos < -50 ) pos = -50;
      oglmgr->setPosition( pos );
      Invalidate( FALSE );
    }
 
  CView::OnMouseMove(nFlags, point);
}

afx_msg void oglView::OnMButtonDblClk( UINT nFlags, CPoint point )
{
  char msg[ 256 ];
  sprintf( msg, "Pos %f", oglmgr->getPosition(), GetLastError() );
  MessageBox( msg, NULL, MB_ICONINFORMATION | MB_OK );
}
