// MFCViewerView.cpp : implementation of the CMFCViewerView class
//

#include "stdafx.h"
#include "MFCViewer.h"

#include "MFCViewerDoc.h"
#include "MFCViewerView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
//Bruno Stefanizzi (bstefani@paris.sgi.com)
/////////////////////////////////////////////////////////////////////////////
// CMFCViewerView

IMPLEMENT_DYNCREATE(CMFCViewerView, CView)

BEGIN_MESSAGE_MAP(CMFCViewerView, CView)
	//{{AFX_MSG_MAP(CMFCViewerView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_SIZE()
	ON_WM_ERASEBKGND()
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMFCViewerView construction/destruction

CMFCViewerView::CMFCViewerView()
{
	// TODO: add construction code here
}

CMFCViewerView::~CMFCViewerView()
{
}

BOOL CMFCViewerView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs
	cs.style |= (WS_CLIPCHILDREN | WS_CLIPSIBLINGS | CS_OWNDC);

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMFCViewerView drawing

void CMFCViewerView::OnDraw(CDC* pDC)
{
	CMFCViewerDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here
	paint();
}

/////////////////////////////////////////////////////////////////////////////
// CMFCViewerView diagnostics

#ifdef _DEBUG
void CMFCViewerView::AssertValid() const
{
	CView::AssertValid();
}

void CMFCViewerView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CMFCViewerDoc* CMFCViewerView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMFCViewerDoc)));
	return (CMFCViewerDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMFCViewerView message handlers

int CMFCViewerView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// TODO: Add your specialized creation code here
	int nPixelFormat;					// Pixel format index

	m_hDC = ::GetDC(m_hWnd);			// Get the Device context

	static PIXELFORMATDESCRIPTOR pfd = {
		sizeof(PIXELFORMATDESCRIPTOR),	// Size of this structure
		1,								// Version of this structure	
		PFD_DRAW_TO_WINDOW |			// Draw to Window (not to bitmap)
		PFD_SUPPORT_OPENGL |			// Support OpenGL calls in window
		PFD_DOUBLEBUFFER,				// Double buffered mode
		PFD_TYPE_RGBA,					// RGBA Color mode
		24,								// Want 24bit color 
		0,0,0,0,0,0,					// Not used to select mode
		0,0,							// Not used to select mode
		0,0,0,0,0,						// Not used to select mode
		32,								// Size of depth buffer
		0,								// Not used to select mode
		0,								// Not used to select mode
		PFD_MAIN_PLANE,					// Draw in main plane
		0,								// Not used to select mode
		0,0,0 };						// Not used to select mode

	// Choose a pixel format that best matches that described in pfd
	nPixelFormat = ChoosePixelFormat(m_hDC, &pfd);

	// Set the pixel format for the device context
	VERIFY(SetPixelFormat(m_hDC, nPixelFormat, &pfd));

	InitializePalette();

	// Create the rendering context
    theContext = new csContext(m_hDC);
	theContext->ref();
    theContext->makeCurrent(m_hDC);
	thePose = new csTransform;
	theRoot.addChild(thePose);
	theBall  = new trackball(thePose);
	InitializeScene();
	
	return 0;
}

void CMFCViewerView::OnDestroy() 
{
	CView::OnDestroy();
	theContext->unref();
	// TODO: Add your message handler code here
	
}

void CMFCViewerView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	// TODO: Add your message handler code here
	theContext->makeCurrent(m_hDC);
  
	width = cx;
	height = cy;
	theContext->setViewport( 0, 0, cx, cy );
#ifdef OPDRAW
	theDraw->setWindowSize( cx, cy );
#endif
  if (width > height)
  {
	theCam->setHorizFOV( CS_DEG2RAD(45) );
	theCam->setVertFOV(CS_DEG2RAD(45)*((float)height/(float)width) );
  }
  else
  {
    theCam->setVertFOV(CS_DEG2RAD(45));
    theCam->setHorizFOV(CS_DEG2RAD(45)*((float)width/(float)height));
  }

}

BOOL CMFCViewerView::OnEraseBkgnd(CDC* pDC) 
{
	// TODO: Add your message handler code here and/or call default
	
	return FALSE;
}

void CMFCViewerView::InitializePalette()
{
	PIXELFORMATDESCRIPTOR pfd;	// Pixel Format Descriptor
	LOGPALETTE *pPal;			// Pointer to memory for logical palette
	int nPixelFormat;			// Pixel format index
	int nColors;				// Number of entries in palette
	int i;						// Counting variable
	BYTE RedRange,GreenRange,BlueRange;
								// Range for each color entry (7,7,and 3)

	// Get the pixel format index and retrieve the pixel format description
	nPixelFormat = GetPixelFormat(m_hDC);
	DescribePixelFormat(m_hDC, nPixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &pfd);

	// Does this pixel format require a palette?  If not, do not create a
	// palette and just return NULL
	if(!(pfd.dwFlags & PFD_NEED_PALETTE))
		return;

	// Number of entries in palette.  8 bits yeilds 256 entries
	nColors = 1 << pfd.cColorBits;	

	// Allocate space for a logical palette structure plus all the palette entries
	pPal = (LOGPALETTE*)malloc(sizeof(LOGPALETTE) +nColors*sizeof(PALETTEENTRY));

	// Fill in palette header 
	pPal->palVersion = 0x300;		// Windows 3.0
	pPal->palNumEntries = nColors; // table size

	// Build mask of all 1's.  This creates a number represented by having
	// the low order x bits set, where x = pfd.cRedBits, pfd.cGreenBits, and
	// pfd.cBlueBits.  
	RedRange = (1 << pfd.cRedBits) -1;
	GreenRange = (1 << pfd.cGreenBits) - 1;
	BlueRange = (1 << pfd.cBlueBits) -1;

	// Loop through all the palette entries
	for(i = 0; i < nColors; i++)
		{
		// Fill in the 8-bit equivalents for each component
		pPal->palPalEntry[i].peRed = (i >> pfd.cRedShift) & RedRange;
		pPal->palPalEntry[i].peRed = (unsigned char)(
			(double) pPal->palPalEntry[i].peRed * 255.0 / RedRange);

		pPal->palPalEntry[i].peGreen = (i >> pfd.cGreenShift) & GreenRange;
		pPal->palPalEntry[i].peGreen = (unsigned char)(
			(double)pPal->palPalEntry[i].peGreen * 255.0 / GreenRange);

		pPal->palPalEntry[i].peBlue = (i >> pfd.cBlueShift) & BlueRange;
		pPal->palPalEntry[i].peBlue = (unsigned char)(
			(double)pPal->palPalEntry[i].peBlue * 255.0 / BlueRange);

		pPal->palPalEntry[i].peFlags = (unsigned char) NULL;
		}
		
	// Create the palette
	m_GLPalette.CreatePalette(pPal);

	// Go ahead and select and realize the palette for this device context
	SelectPalette(m_hDC,(HPALETTE)m_GLPalette,FALSE);
	RealizePalette(m_hDC);

	// Free the memory used for the logical palette structure
	free(pPal);
}

void CMFCViewerView::InitializeScene()
{
	theCam  = new csPerspCamera;
#ifdef OPDRAW
	theDraw = new opDrawAction(2, false);
#else
	theDraw = new csDrawAction();
#endif
	theDraw->ref();
	RECT rectan;
	GetClientRect(&rectan);
	width = rectan.right - rectan.left;
	height = rectan.bottom - rectan.top;
  
  // DrawAction setup
  theDraw->setCamera( theCam );
#ifdef OPDRAW
  theDraw->setWindowSize( width, height );
#endif

  theLightArray    = new csLight* [2];

  theLightArray[0] = new csDirectionalLight();
  theLightArray[0]->ref();
  ((csDirectionalLight*)theLightArray[0])->setDirection(0.0f,0.0f,-1.0f);
  ((csDirectionalLight*)theLightArray[0])->setAmbientIntensity( 0.2f );
  ((csDirectionalLight*)theLightArray[0])->setIntensity( 1.0f );
  theLightArray[0]->setOn(true);

  theLightArray[1] = new csDirectionalLight();
  theLightArray[1]->ref();
  ((csDirectionalLight*)theLightArray[1])->setDirection(0.0f,0.0f,1.0f);
  ((csDirectionalLight*)theLightArray[1])->setAmbientIntensity( 0.2f );
  ((csDirectionalLight*)theLightArray[1])->setIntensity( 1.0f );
  theLightArray[1]->setOn(true);
	theRoot.light()->append(theLightArray[0]);
	theRoot.light()->append(theLightArray[1]);
#ifdef OPDRAW
  theDraw->setLights( 2, theLightArray );
  theDraw->setVFCullMode(true);
  theDraw->setConservativeMode(false);
  theDraw->setDrawCulledMode(false);
  theDraw->setOCCullMode(true);
//  theDraw->setClipVolumeHintMode(false);
  theDraw->setClearColor( csVec4f( 0.05f, 0.0f, 0.05f, 1.0f ) );
#endif

  theContext->setViewport( 0, 0, width, height );
  theContext->setLightEnable( true );
  theContext->setShadeModel( csContext::SMOOTH_SHADE );
  theContext->setDepthFunc( csContext::LEQUAL_DFUNC );
  theContext->setDepthEnable(true);

  glEnable(GL_POLYGON_OFFSET_LINE);
  //glEnable(GL_POLYGON_OFFSET_FILL);
  glPolygonOffset(0.1, 0.5);
//  const unsigned char *c = glGetString(GL_EXTENSIONS);
//  TRACE("%s\n",c);
  ////////////////////  Draw  ////////////////////  
}

void CMFCViewerView::setClipPlanes()
{
  csSphereBound sphere;
  thePose->getSphereBound(sphere);
  float d = sphere.center.length();

  float nearVal = d-sphere.radius*1.2f;  
  float farVal  = d+sphere.radius*1.2f;
  
  if (nearVal <1.0f)
    nearVal = sphere.radius/100.0f;

  if (farVal<nearVal)
    farVal = nearVal+sphere.radius*1.2f;

  theCam->setNearClip(nearVal);
  theCam->setFarClip(farVal);

}

void CMFCViewerView::resizeToFit(csGroup * node)
{
  csSphereBound sphere;
  csSphereBound *sp = &sphere;
  node->getSphereBound(sphere);
  csBoxBound bbox;
  bbox.around((const csSphereBound **)&sp,1);
  
  csVec3f delta;
  delta.sub(bbox.max,bbox.min);
  delta[2] = 0.0f;
  float zTrans = delta.length() * 1.0f;
  
  thePose->setTranslation( -sphere.center[0], -sphere.center[1], -sphere.center[2]-zTrans);
  thePose->setCenter(sphere.center[0], sphere.center[1],sphere.center[2] );
  setClipPlanes();
}



void CMFCViewerView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	
	CView::OnLButtonDown(nFlags, point);
	_mousex = point.x;
	_mousey = point.y;
/*
	if ((nFlags & MK_SHIFT))
	{
		csHit results;
		theCam->pick( theRoot, _mousex, height-_mousey, results );
        if (results.shape != NULL)
		{
			BoundBox->selectednode = results.shape;
			BoundBox->nodeType = BoundBox->selectednode->getType();
		}
		else
		{
			BoundBox->selectednode = NULL;
		}
		COmaDoc* pDoc = GetDocument();
		pDoc->selectednodeChanged = true;
		pDoc->UpdateAllViews(NULL);
		if (pDoc->m_pDlg2D->GetSafeHwnd() != 0)
		{
			pDoc->m_pDlg2D->RedrawWindow();
		}
	}
*/
}

void CMFCViewerView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	
	CView::OnRButtonDown(nFlags, point);

	_zoom = point.y;
      csVec3f pos;
      thePose->getTranslation( pos );
      _startz = pos[2];
}

void CMFCViewerView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	need_refresh = NO_REFRESH;
	CView::OnMouseMove(nFlags, point);

if ( nFlags & MK_LBUTTON ){
      theBall->updateTrackball( point.x, point.y,
			      point.x-_mousex, point.y-_mousey,
			      width, height );
      
      _mousex = point.x;
      _mousey = point.y;
      need_refresh = MVT_REFRESH;
    }
    else if ( nFlags & MK_RBUTTON ) {
      float delta = ( point.y - _zoom ) / float( height );
      csSphereBound sph;
      theUserRoot->getSphereBound( sph );
      csVec3f pos;
      thePose->getTranslation( pos );
      pos[2] = _startz + delta*sph.radius;
      thePose->setTranslation( pos );
	  setClipPlanes();
      need_refresh = MVT_REFRESH;
    }

	if (need_refresh != NO_REFRESH)
	{
		paint();
	}
    /*
    else if ( mevent->state & Button3Mask ) {
      float deltax = ( mevent->x - _transx ) / float( _info->width );
      float deltay = ( _transy - mevent->y ) / float( _info->height );

      csSphereBound sph;
      _info->root->getSphereBound( sph );
      csVec3f pos;

      _info->pose->getTranslation( pos );

      pos[0] = _startx + deltax*sph.radius;
      pos[1] = _starty + deltay*sph.radius;

      _info->pose->setTranslation( pos );

      need_refresh = MVT_REFRESH;
	  */
}

void CMFCViewerView::paint()
{
#ifndef OPDRAW
	theContext->clear(csContext::COLOR_CLEAR | csContext::DEPTH_CLEAR,
      0.1f, 0.0f, 0.1f, 1.0f );
	theCam->draw(theDraw);
#endif
		theDraw->apply(&theRoot);
#ifdef OPDRAW
		theDraw->postDraw();
#endif
	SwapBuffers(m_hDC);

}

void CMFCViewerView::NewScene()
{
  theUserRoot = GetDocument()->theUserRoot;
  int nkids = thePose->getChildCount();
 	while ( nkids )
		{
			thePose->removeChild(nkids-1);
			nkids--;
		}
  thePose->addChild(theUserRoot);
  
  resizeToFit(theUserRoot);
#ifdef OPDRAW
  theDraw->setScene(&theRoot);
  theDraw->initializeScene();	
#else
#endif
}


void CMFCViewerView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
		paint();
}

void CMFCViewerView::OnInitialUpdate() 
{
	NewScene();			
	CView::OnInitialUpdate();

	// TODO: Add your specialized code here and/or call the base class
}
