// MultiOGLView.cpp : implementation of the CMultiOGLView class
//

#include "stdafx.h"
#include "MultiOGL.h"

#include "MultiOGLDoc.h"
#include "MultiOGLView.h"

#include <math.h>

#include <stdlib.h>
#include <stdio.h>
#include <io.h>
#include <fcntl.h>
#include <assert.h>
#include <windows.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#ifdef _DEBUG
#define DEBUG_PRINT(_xxx) printf(_xxx)
#else
#define DEBUG_PRINT(_xxx)
#endif

float PI = 3.1415926535897932;   // PI is not defined in math.h

/////////////////////////////////////////////////////////////////////////////
// SetupConsole
//
// Make sure there is a console to print messages to
//
/////////////////////////////////////////////////////////////////////////////

void SetupConsole()
{
	int hCrt;
	FILE *hf;
	static int initialized = 0;

	if(initialized == 1)
		{
		DEBUG_PRINT("SetupConsole only needs to be called once\n");
		return;
		}

	AllocConsole();

	// Setup stdout
	hCrt = _open_osfhandle( (long)GetStdHandle(STD_OUTPUT_HANDLE), _O_TEXT );
	hf = _fdopen(hCrt, "w" );
	*stdout = *hf;
	setvbuf(stdout, NULL, _IONBF, 0);

	// Setup stderr
	hCrt = _open_osfhandle( (long)GetStdHandle(STD_ERROR_HANDLE), _O_TEXT );
	hf = _fdopen(hCrt, "w" );
	*stderr = *hf;
	setvbuf(stderr, NULL, _IONBF, 0);

	initialized = 1;
}

/////////////////////////////////////////////////////////////////////////////
// CMultiOGLView

IMPLEMENT_DYNCREATE(CMultiOGLView, CView)

BEGIN_MESSAGE_MAP(CMultiOGLView, CView)
	//{{AFX_MSG_MAP(CMultiOGLView)
	ON_WM_CREATE()
	ON_WM_ERASEBKGND()
	ON_WM_DESTROY()
	ON_WM_SIZE()
	ON_WM_SETFOCUS()
	ON_WM_KILLFOCUS()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

       
/////////////////////////////////////////////////////////////////////////////
// CMultiOGLView construction/destruction

CMultiOGLView::CMultiOGLView()
{
	// TODO: add construction code here
	DEBUG_PRINT("CMultiOGLView::CMultiOGLView\n");
	SetupConsole();

}

CMultiOGLView::~CMultiOGLView()
{
	DEBUG_PRINT("CMultiOGLView::~CMultiOGLView\n");
}

BOOL CMultiOGLView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs
	DEBUG_PRINT("CMultiOGLView::PreCreateWindow\n");
	
    cs.style |= (WS_CLIPCHILDREN |WS_CLIPSIBLINGS | CS_OWNDC);

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMultiOGLView drawing

void CMultiOGLView::OnDraw(CDC* pDC)
{

	CMultiOGLDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here
	DEBUG_PRINT("MultiOGLView::OnDraw\n");

	if (m_pOGLRenderThread == NULL) 
	{
		m_pOGLRenderThread = AfxBeginThread(OGLRenderThreadProc, &m_OGLRenderThreadInfo);
	}
 
}

/////////////////////////////////////////////////////////////////////////////
// CMultiOGLView diagnostics

#ifdef _DEBUG
void CMultiOGLView::AssertValid() const
{
	CView::AssertValid();
}

void CMultiOGLView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CMultiOGLDoc* CMultiOGLView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMultiOGLDoc)));
	return (CMultiOGLDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMultiOGLView message handlers

int CMultiOGLView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// TODO: Add your specialized creation code here
	DEBUG_PRINT("MultiOGLView::OnCreate\n");

	PIXELFORMATDESCRIPTOR	pfd;
	int	pixelFormat;

	pfd.nSize =					sizeof(PIXELFORMATDESCRIPTOR);
	pfd.nVersion = 			1;
	pfd.dwFlags =				PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL |
									PFD_TYPE_RGBA | PFD_DOUBLEBUFFER;
	pfd.cColorBits =			24;
	pfd.cRedBits =				0;
	pfd.cRedShift =			0;
	pfd.cGreenBits =			0;
	pfd.cGreenShift =			0;
	pfd.cBlueBits =			0;
	pfd.cBlueShift =			0;
	pfd.cAlphaBits =			0;
	pfd.cAlphaShift =			0;
	pfd.cAccumBits =			0;
	pfd.cAccumRedBits =		0;
	pfd.cAccumGreenBits =	0;
	pfd.cAccumBlueBits =		0;
	pfd.cAccumAlphaBits =	0;
	pfd.cDepthBits =			8;
	pfd.cStencilBits =		0;
	pfd.cAuxBuffers =			0;
	pfd.iLayerType =			PFD_MAIN_PLANE;
	pfd.bReserved =			0;
	pfd.dwLayerMask =			0;
	pfd.dwVisibleMask =		0;
	pfd.dwDamageMask =		0;
 
	m_hDC = ::GetDC(m_hWnd);
        

    pixelFormat = ChoosePixelFormat(m_hDC, &pfd);
        
    DescribePixelFormat(m_hDC, pixelFormat,
                sizeof(PIXELFORMATDESCRIPTOR),
                &pfd);
        
    if(SetPixelFormat(m_hDC, pixelFormat, &pfd) == FALSE)
            exit(1);
        
    m_hRC = wglCreateContext(m_hDC);

    wglMakeCurrent( m_hDC, m_hRC );
        
    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
    glEnable(GL_NORMALIZE);
    glEnable(GL_LIGHT0);
    glEnable(GL_LIGHTING);

    glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
    glEnable(GL_COLOR_MATERIAL);

    glEnable(GL_DEPTH_TEST);

    glMatrixMode(GL_PROJECTION);
    glFrustum(-1, 1, -1, 1, 1, 3);
    glMatrixMode(GL_MODELVIEW);
    glTranslatef(0, 0, -2);

    glClearColor(0.2, 0.2, 0.2, 1.0);

    wglMakeCurrent( NULL, NULL );

	// set up m_OGLRenderThreadInfo structure
	m_hEventOGLRenderThreadKilled = CreateEvent(NULL, FALSE, FALSE, NULL);	
	m_hEventOGLRenderThreadKill = CreateEvent(NULL, FALSE, FALSE, NULL);
	m_hEventOGLRenderSetHighlight = CreateEvent(NULL, FALSE, TRUE, NULL);
	m_hEventOGLRenderKillHighlight = CreateEvent(NULL, FALSE, FALSE, NULL);
	m_hEventOGLRenderOnSize = CreateEvent(NULL, FALSE, FALSE, NULL);

	m_OGLRenderThreadInfo.m_hEventOGLRenderThreadKilled = m_hEventOGLRenderThreadKilled;	
	m_OGLRenderThreadInfo.m_hEventOGLRenderThreadKill = m_hEventOGLRenderThreadKill;
	m_OGLRenderThreadInfo.m_hEventOGLRenderSetHighlight = m_hEventOGLRenderSetHighlight;
	m_OGLRenderThreadInfo.m_hEventOGLRenderKillHighlight = m_hEventOGLRenderKillHighlight;
	m_OGLRenderThreadInfo.m_hEventOGLRenderOnSize = m_hEventOGLRenderOnSize;

	width = 0;
	height = 0;
	m_OGLRenderThreadInfo.width = width;
	m_OGLRenderThreadInfo.height = height;
	
	m_OGLRenderThreadInfo.m_hWnd = m_hWnd;
	m_OGLRenderThreadInfo.m_hDC = m_hDC;
	m_OGLRenderThreadInfo.m_hRC = m_hRC;
	m_pOGLRenderThread = NULL;

	return 0;
}

BOOL CMultiOGLView::OnEraseBkgnd(CDC* pDC) 
{
	// TODO: Add your message handler code here and/or call default
	
	//return CView::OnEraseBkgnd(pDC);
	DEBUG_PRINT("MultiOGLView::OnEraseBkgnd\n");
	return 1;
}

void CMultiOGLView::OnDestroy() 
{
	CView::OnDestroy();
	
	// TODO: Add your message handler code here
	DEBUG_PRINT("MultiOGLView::OnDestroy\n");

	SetEvent(m_hEventOGLRenderThreadKill);

    wglMakeCurrent( NULL, NULL );
    wglDeleteContext( m_hRC );
    ::ReleaseDC( m_hWnd, m_hDC );
	
}

void CMultiOGLView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	

	// TODO: Add your message handler code here
	DEBUG_PRINT("MultiOGLView::OnSize\n");

	m_OGLRenderThreadInfo.width =cx;
	m_OGLRenderThreadInfo.height = cy;
	SetEvent(m_hEventOGLRenderOnSize);
}


void CMultiOGLView::OnSetFocus(CWnd* pOldWnd) 
{
	CView::OnSetFocus(pOldWnd);
	
	// TODO: Add your message handler code here
	DEBUG_PRINT("MultiOGLView::OnSetFocus\n");

	ResetEvent(m_hEventOGLRenderKillHighlight);
	SetEvent(m_hEventOGLRenderSetHighlight);
}

void CMultiOGLView::OnKillFocus(CWnd* pNewWnd) 
{
	CView::OnKillFocus(pNewWnd);
	
	// TODO: Add your message handler code here
	DEBUG_PRINT("MultiOGLView::OnKillFocus\n");	

	ResetEvent(m_hEventOGLRenderSetHighlight);
	SetEvent(m_hEventOGLRenderKillHighlight);
}

UINT OGLRenderThreadProc(LPVOID pParam)
{
	COGLRenderThreadInfo * pOGLRenderInfo = (COGLRenderThreadInfo *) pParam;

	int count = 0;
	BOOL highlight = FALSE;

	DEBUG_PRINT("OGLRenderThreadProc begin\n");
    
	wglMakeCurrent(pOGLRenderInfo->m_hDC, pOGLRenderInfo->m_hRC);
	
	while (TRUE)
	{

		if (WaitForSingleObject(pOGLRenderInfo->m_hEventOGLRenderThreadKill, 0)
			== WAIT_OBJECT_0)
		{
			DEBUG_PRINT("m_hEventOGLRenderThreadKill\n");
			break;
		}


		if (WaitForSingleObject(pOGLRenderInfo->m_hEventOGLRenderSetHighlight, 0)
			== WAIT_OBJECT_0)
		{
			highlight = TRUE;
			ResetEvent(pOGLRenderInfo->m_hEventOGLRenderSetHighlight);
		}

		if (WaitForSingleObject(pOGLRenderInfo->m_hEventOGLRenderKillHighlight, 0)
			== WAIT_OBJECT_0)
		{					
			highlight = FALSE;			
			ResetEvent(pOGLRenderInfo->m_hEventOGLRenderKillHighlight);
		}

		if (WaitForSingleObject(pOGLRenderInfo->m_hEventOGLRenderOnSize, 0)
			== WAIT_OBJECT_0)
		{			
			glViewport( 0, 0, pOGLRenderInfo->width, pOGLRenderInfo->height );
			ResetEvent(pOGLRenderInfo->m_hEventOGLRenderOnSize);
		}


		GLfloat rotx = 5.0 * count;
		GLfloat roty = 7.0 * count;
		GLfloat rotz = 11.0 * count;

		if (rotx > 360) rotx -= 360;
		if (roty > 360) roty -= 360;
		if (rotz > 360) rotz -= 360;

		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		glPushMatrix();
		glRotatef(rotx, 1, 0, 0);
		glRotatef(roty, 0, 1, 0);
		glRotatef(rotz, 0, 0, 1);
		{
			GLfloat radius = 0.60;
			int s, numSides = 32;

			if (highlight) {
				glColor3f(0.1, 0.5, 1.6);
			} else {
			  glColor3f(0.8, 0.5, 0.1);
			}

			glBegin(GL_TRIANGLE_STRIP);
			for (s=0; s<numSides+1; ++s) {
			  double a = s * (2 * PI / numSides);
				GLfloat x = radius * (GLfloat) cos(a);
			   GLfloat y = radius * (GLfloat) sin(a);
			   glNormal3f(x, y, 0.0);
			   glVertex3f(x, y, 0.60);
			   glVertex3f(x, y,-0.60);
			}
			glEnd();
		}
		glPopMatrix();

		count++;
 
		SwapBuffers(pOGLRenderInfo->m_hDC);
 
		ValidateRect(pOGLRenderInfo->m_hWnd, NULL);
	}

	SetEvent(pOGLRenderInfo->m_hEventOGLRenderThreadKilled);

    wglMakeCurrent( NULL, NULL );

	DEBUG_PRINT("OGLRenderThreadProc return\n");
 
	return 0;
}
