/*
 * Copyright (c) 1999 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <iostream.h>
#include <stdlib.h>
#include <gl/glut.h>

#include <windows.h>

/////////////////////////////////////////////////////////////////////////////
//
// function prototypes
//
/////////////////////////////////////////////////////////////////////////////

void init();
void draw();
void idle();
void reshape( int ww, int hh );
void key( unsigned char key, int x, int y );

int fontsize = 44;
int listbase_bitmap;
int listbase_outline;

/////////////////////////////////////////////////////////////////////////////
//
// ogl initialization
//
/////////////////////////////////////////////////////////////////////////////

static void init()
{
  glClearColor( 0, 0, 0, 0 );

  HDC hdc = wglGetCurrentDC();

  // setup font basics
  HFONT hfont;
  LOGFONT logfont;
  memset( &logfont, 0, sizeof( logfont ) ); 
  strcpy( logfont.lfFaceName, "Impact" ); 

  // create a bitmap font
  logfont.lfHeight = fontsize;
  hfont = CreateFontIndirect( &logfont );
  SelectObject( hdc, hfont );
  listbase_bitmap = glGenLists( 255 );
  bool result_bitmap = (bool) wglUseFontBitmaps( hdc, 0, 255, listbase_bitmap );

  
  // create an outline (3d) font
  logfont.lfHeight = 1;
  hfont = CreateFontIndirect( &logfont );
  SelectObject( hdc, hfont );
  listbase_outline = glGenLists( 255 );
  bool result_outline = (bool) wglUseFontOutlines( hdc, 0, 255, listbase_outline,
					   0.0, 1.0, WGL_FONT_POLYGONS, NULL );
}

/////////////////////////////////////////////////////////////////////////////
//
// drawing / resizing functions
//
/////////////////////////////////////////////////////////////////////////////

static void reshape( int ww, int hh )
{
  glViewport( 0, 0, ww, hh );

  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();

  glOrtho( 0.0, 1.0, 0.0, 1.0, -10.0, 10.0 );

  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity();
}

static void draw()
{
  // clear fb
  glClear( GL_COLOR_BUFFER_BIT );

  // reset matrix stack
  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity();

  // draw bw quad
  glBegin( GL_TRIANGLES );

  glColor3f( 1.0, 1.0, 1.0 );
  glVertex3f( 0.0, 0.0, 0.0 );
  glVertex3f( 1.0, 0.0, 0.0 );
  glVertex3f( 0.0, 1.0, 0.0 );

  glEnd();

  // draw ogl text
  char txt[] = "OpenGL";
  int nchars = ( sizeof( txt ) / sizeof( txt[ 0 ] ) ) - 1;

  glColor3f( 1.0, 0.0, 0.0 );
  glRasterPos3f( .11, .2, 0.0 );
  glListBase( listbase_bitmap );
  glCallLists( nchars, GL_UNSIGNED_BYTE, txt );

  glRasterPos3f( .66, .7, 0.0 );
  glListBase( listbase_bitmap );
  glCallLists( nchars, GL_UNSIGNED_BYTE, txt );

  glTranslatef( .4, .6, 0.0 );
  glRotatef( -45, 0, 0, 1 );
  glScalef( .15, .15, .15 );

  glColor3f( 0.0, 0.0, 1.0 );
  glListBase( listbase_outline );
  glCallLists( nchars, GL_UNSIGNED_BYTE, txt );

  // end frame & swap
  glutSwapBuffers();
}

/////////////////////////////////////////////////////////////////////////////
//
// key-event handler
//
/////////////////////////////////////////////////////////////////////////////

static void key( unsigned char key, int xx, int yy )
{
  switch( key )
    {
    case 'q':
    case 27:
      {
	exit(0);
      }
    }
}

/////////////////////////////////////////////////////////////////////////////
//
// main (setup, visual selection, callback registration)
//
/////////////////////////////////////////////////////////////////////////////

int main(int argc, char **argv)
{
  glutInit( &argc, argv );
  
  glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE );
  glutInitWindowSize( 500, 500 );
  glutCreateWindow( "glut test shell" );
  
  init();
  
  glutReshapeFunc( reshape );
  glutKeyboardFunc( key );
  glutDisplayFunc( draw );
  glutMainLoop();

  return( 1 );
}
