/*
 * Copyright 1999 Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED
 *
 * You may use and modify this software and its documentation for your own
 * internal purposes without fee, provided that you agree that you will not
 * remove this notice from this software under any circumstances.  You may not
 * disclose the contents of this file to third parties in any form, in whole
 * or in part, without the prior written permission of Silicon Graphics, Inc.
 *
 * THIS SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT ANY WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
 * OF MERCHANTABLIITY OR FITNESS FOR A PARTICULAR PURPOSE.
 * In no event shall Silicon Graphics be liable for any special, incidental,
 * indirect or consequential damages of any kind, or any damages whatsoever
 * resulting from loss of use, data or profits, whether or not advised of the
 * possibility of damage, and on any theory of liability, arising out of or
 * in connection with the possession, use or performance of this software.
 *
 * U.S. GOVERNMENT RESTRICTED RIGHTS LEGEND
 * Use, duplication, or disclosure by the Government is subject to restrictions
 * set forth in FAR 52.227.19(c)(2) or subparagraph (c)(1)(ii) of the Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7013 and/or in
 * similar or successor clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the United States.
 * Contractor/manufacturer is Silicon Graphics, Inc., 2011 N. Shoreline Blvd.,
 * Mountain View, CA 94039-7311.
 *
 * Author: Jacques Lefaucheux   jacques@paris.sgi.com
 * January 20, 1999
 */

#include "overlays.h"
#include <stdio.h>
// pragmas
#pragma warning(disable : 4244)		// disable conversion warnings


// overlays palettes for colorindex

COLORREF crEntries[MAX_OVERLAY_COLORS] = {
      0x00000000,			// black (ref #0 = transparent)
      0x000000ff,			// red
	  0x0000ff00,			// green
	  0x00ff0000,			// blue
	  0x00ff00ff,			// magenta
	  0x0000ffff,			// yellow
	  0x00ffff00,			// cyan
	  0x00ffffff,			// white
	  0x00000000			// black
    };

// create main context and overlay context
int createContextAndOverlays(HDC hDC,HGLRC *ctx,HGLRC *ovlctx,DWORD pfdflags){
    int pf, maxpf;
	int layer=0;
	DWORD flags;
    PIXELFORMATDESCRIPTOR pfd;
    LAYERPLANEDESCRIPTOR  lpd;		/* layer plane descriptor */
   
	// init
	*ctx    = 0;
	*ovlctx = 0;

	flags = OVL_SINGLE|OVL_COLORINDEX;
	if (pfdflags == 0) pfdflags = ~0;

    // search for overlays planes
    maxpf = DescribePixelFormat(hDC, 0, 0, NULL);

	for(pf = 1; pf <= maxpf; pf++) {
		DescribePixelFormat(hDC, pf, sizeof(PIXELFORMATDESCRIPTOR), &pfd);

		// the bReserved field of the PIXELFORMATDESCRIPTOR contains the
		// number of overlay/underlay planes
		if (pfd.bReserved > 0) {
			wglDescribeLayerPlane(hDC, pf, pfd.bReserved,
				sizeof(LAYERPLANEDESCRIPTOR),&lpd);

			if ((lpd.dwFlags & LPD_SUPPORT_OPENGL) && (lpd.iPixelType == LPD_TYPE_COLORINDEX)){
				if (lpd.dwFlags & LPD_DOUBLEBUFFER)
					continue;
			// now check the "normal" pixel format descriptor for the layer
			DescribePixelFormat(hDC, pf, sizeof(PIXELFORMATDESCRIPTOR), &pfd);
#ifdef DEBUG_OVL
			dumpNormals(hDC, pf);
#endif
			if (pfd.dwFlags & pfdflags) {
				layer = pfd.bReserved;
				break;
			}
			}
		}
	}

	if (layer == 0) {
		// couldn't find any overlay/underlay planes
		MessageBox(NULL,
	       "createContextAndOverlays: Fatal Error:\nHardware does not support this normals/overlay planes combinaison",
	       "Error", MB_OK);
		return 0;
	}

    // set the pixel format

    if( SetPixelFormat(hDC, pf, &pfd) == FALSE ) {
		MessageBox(NULL,
				   "createContextAndOverlays: Fatal Error:  Cannot set format specified.",
				   "Error", MB_OK);
		return 0;
		}
    
    // set up the layer palette for overlays 
    if (lpd.iPixelType & LPD_TYPE_COLORINDEX){
		wglSetLayerPaletteEntries(hDC, layer, 0, MAX_OVERLAY_COLORS, crEntries);
		wglRealizeLayerPalette(hDC, layer, TRUE);
	}

    // debug infos
#ifdef DEBUG_OVL
	dumpOverlays(hDC, pf, layer);
#endif

	// create an OpenGL context
    *ctx = wglCreateContext(hDC);

    // create an OpenGL overlay context
    *ovlctx = wglCreateLayerContext(hDC, layer);

    return pf;
}    

#ifdef DEBUG_OVL
void dumpOverlays(HDC hDC,int i,int n){
     LAYERPLANEDESCRIPTOR  lpd;
     wglDescribeLayerPlane(hDC, i, n,sizeof(LAYERPLANEDESCRIPTOR), &lpd);

	 printf("Dump Overlays Infos: PixelFormat %d Number = %d\n", i,n);
	 printf("\tSupport OpenGL: %s\n",(lpd.dwFlags & LPD_SUPPORT_OPENGL)?"Yes":"No");
	 printf("\tSupport GDI   : %s\n",(lpd.dwFlags & LPD_SUPPORT_GDI)?"Yes":"No");
	 printf("\tDoubleBuffer  : %s\n",(lpd.dwFlags & LPD_DOUBLEBUFFER)?"Yes":"No");
	 printf("\tStereo        : %s\n",(lpd.dwFlags & LPD_STEREO)?"Yes":"No");
	 printf("\tSwapExchange  : %s\n",(lpd.dwFlags & LPD_SWAP_EXCHANGE)?"Yes":"No");
	 printf("\tSwapCopy      : %s\n",(lpd.dwFlags & LPD_SWAP_COPY)?"Yes":"No");
	 printf("\tTransparent   : %s\n",(lpd.dwFlags & LPD_TRANSPARENT)?"Yes":"No");
	 if (lpd.dwFlags & LPD_TRANSPARENT)
		printf("\tPixel Transparent: 0x%x \n", lpd.crTransparent);
	 printf("\tShare Depth   : %s\n",(lpd.dwFlags & LPD_SHARE_DEPTH)?"Yes":"No");
	 printf("\tShare Stencil : %s\n",(lpd.dwFlags & LPD_SHARE_STENCIL)?"Yes":"No");
	 printf("\tShare Accum   : %s\n",(lpd.dwFlags & LPD_SHARE_ACCUM)?"Yes":"No");
	 printf("\tPixel Type    : %s\n",(lpd.iPixelType == LPD_TYPE_RGBA)?"RGBA":"COLORINDEX");
     printf("\tColor Planes  : %d\n", lpd.cColorBits);

	 if (lpd.iPixelType == LPD_TYPE_RGBA){
		printf("\t\tRed   Bits/Shift  : %d %d\n", lpd.cRedBits , lpd.cRedShift);
		printf("\t\tGreen Bits/Shift  : %d %d\n", lpd.cGreenBits , lpd.cGreenShift);
		printf("\t\tBlue  Bits/Shift  : %d %d\n", lpd.cBlueBits , lpd.cBlueShift);
		printf("\t\tAlpha Bits/Shift  : %d %d\n", lpd.cAlphaBits , lpd.cAlphaShift);
	 }
}
void dumpNormals(HDC hDC,int i){
     PIXELFORMATDESCRIPTOR  pfd;
     DescribePixelFormat(hDC, i, sizeof(PIXELFORMATDESCRIPTOR), &pfd);

	 printf("Dump Normals Infos: PixelFormat %d \n", i);
	 printf("\tSupport OpenGL: %s\n",(pfd.dwFlags & PFD_SUPPORT_OPENGL)?"Yes":"No");
	 printf("\tSupport GDI   : %s\n",(pfd.dwFlags & PFD_SUPPORT_GDI)?"Yes":"No");
	 printf("\tDoubleBuffer  : %s\n",(pfd.dwFlags & PFD_DOUBLEBUFFER)?"Yes":"No");
	 printf("\tStereo        : %s\n",(pfd.dwFlags & PFD_STEREO)?"Yes":"No");
	 printf("\tSwapExchange  : %s\n",(pfd.dwFlags & PFD_SWAP_EXCHANGE)?"Yes":"No");
	 printf("\tSwapCopy      : %s\n",(pfd.dwFlags & PFD_SWAP_COPY)?"Yes":"No");
	 printf("\tSwapLayerBuf  : %s\n",(pfd.dwFlags & PFD_SWAP_LAYER_BUFFERS)?"Yes":"No");
	 printf("\tSupport Window: %s\n",(pfd.dwFlags & PFD_DRAW_TO_WINDOW)?"Yes":"No");
	 printf("\tSupport Bitmap: %s\n",(pfd.dwFlags & PFD_DRAW_TO_BITMAP)?"Yes":"No");
	 printf("\tPixel Type    : %s\n",(pfd.iPixelType == PFD_TYPE_RGBA)?"RGBA":"COLORINDEX");
     printf("\tColor Planes  : %d\n", pfd.cColorBits);

	 if (pfd.iPixelType == PFD_TYPE_RGBA){
		printf("\t\tRed   Bits/Shift  : %d %d\n", pfd.cRedBits , pfd.cRedShift);
		printf("\t\tGreen Bits/Shift  : %d %d\n", pfd.cGreenBits , pfd.cGreenShift);
		printf("\t\tBlue  Bits/Shift  : %d %d\n", pfd.cBlueBits , pfd.cBlueShift);
		printf("\t\tAlpha Bits/Shift  : %d %d\n", pfd.cAlphaBits , pfd.cAlphaShift);
	 }
}
#endif