/*
 * Copyright (c) 1991-1998 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <iostream.h>
#include <stdlib.h>
#include <GL/glut.h>

/////////////////////////////////////////////////////////////////////////////
//
// function prototypes
//
/////////////////////////////////////////////////////////////////////////////

void init();
void draw();
void idle();
void reshape( int ww, int hh );
void key( unsigned char key, int x, int y );

/////////////////////////////////////////////////////////////////////////////
//
// ogl initialization
//
/////////////////////////////////////////////////////////////////////////////

static void init()
{
  glClearColor( 0, 0, 0, 0 );
}

/////////////////////////////////////////////////////////////////////////////
//
// drawing / resizing functions
//
/////////////////////////////////////////////////////////////////////////////

static void reshape( int ww, int hh )
{
  glViewport( 0, 0, ww, hh );

  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();

  glOrtho( 0.0, 1.0, 0.0, 1.0, -10.0, 10.0 );

  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity();
}

static void draw()
{
  glClear( GL_COLOR_BUFFER_BIT );

  glBegin( GL_TRIANGLES );

  glColor3f( 1.0, 1.0, 1.0 );
  glVertex3f( 0.0, 0.0, 0.0 );
  glVertex3f( 1.0, 0.0, 0.0 );
  glVertex3f( 0.0, 1.0, 0.0 );

  glEnd();

  glutSwapBuffers();
}

/////////////////////////////////////////////////////////////////////////////
//
// key-event handler
//
/////////////////////////////////////////////////////////////////////////////

static void key( unsigned char key, int xx, int yy )
{
  switch( key )
    {
    case 'q':
    case 27:
      {
	exit(0);
      }
    }
}

/////////////////////////////////////////////////////////////////////////////
//
// main (setup, visual selection, callback registration)
//
/////////////////////////////////////////////////////////////////////////////

int main(int argc, char **argv)
{
  glutInit( &argc, argv );
  
  glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE );
  glutInitWindowSize( 500, 500 );
  glutCreateWindow( "glut test shell" );
  
  init();
  
  glutReshapeFunc( reshape );
  glutKeyboardFunc( key );
  glutDisplayFunc( draw );
  glutMainLoop();

  return( 1 );
}
