/*
 * Copyright (c) 1997, Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#include "config.h"

// Function to get a key value from a file
int GetPrivateProfileString(const char* section, const char* key,
              const char* def, char* buf, size_t len, const char* filename)
{
  FILE *fp;
  int correct_section = 0;
  int seclen = 0;
  int keylen = 0;
  int too_big = 0;
  char holdsec[512] = "\0";
  char holdbuf[512] = "\0";

  if ((fp = fopen(filename, "r")) == NULL) { // file doesn't exist
    if (strlen(def) >= len)
      too_big = 1; // default is bigger than the buffer
    strncpy(buf, def, (too_big) ? len-1 : strlen(def));
    if (too_big)
      strcpy(&buf[(too_big) ? len-1 : strlen(def)], "\0");
    return ((too_big) ? len-1 : strlen(def-1));
  }

  keylen = strlen(key);

  // read file a line at a time no larger than 512 byte lines
  while(fscanf(fp, "%[^\n]", holdbuf) != EOF) {
    fgetc(fp); // get the newline out of the stream

    // we have a new section
    if (0 == strncmp(holdbuf, "[", 1)) {
      if (correct_section) { // we didn't find the key in the correct section
       if (strlen(def) >= len)
         too_big = 1; // default is bigger than the buffer
       strncpy(buf, def, (too_big) ? len-1 : strlen(def));
       if (too_big)
         strcpy(&buf[(too_big) ? len-1 : strlen(def)], "\0");
       fclose(fp);
       return ((too_big) ? len-1 : strlen(def-1)); // return bytes written
      }
      seclen = strlen(holdbuf);
      strncpy(holdsec, &holdbuf[1], seclen-2);
      strcpy(&holdsec[seclen-2], "\0");
      if (0 == strcmp(holdsec, section)) { // we have the correct section
       correct_section = 1;
       continue;
      }
    }
    else if (correct_section){ // we have a key to check
      if (0 == strncmp(holdbuf, key, keylen)) { // we have the correct key
       seclen = strlen(holdbuf)-(keylen);
       if (seclen >= len)
         too_big = 1;
       strncpy(buf, &holdbuf[keylen+1], (too_big) ? len-1 : seclen);
       if (too_big)
         strcpy(&buf[(too_big) ? len-1 : seclen], "\0");
       fclose(fp);
       return ((too_big) ? len-1 : strlen(holdbuf)-(keylen)-1);
      }
      else
       continue;
    }
    else // keep looking for the correct section
      continue;
  }
  // fell out of the loop looks like no matching sections or keys
  // copy the default key in and return
  if (strlen(def) > len)
    too_big = 1;
  strncpy(buf, def, (too_big) ? len-1 : strlen(def));
  if (too_big)
    strcpy(&buf[(too_big) ? len-1 : strlen(def)], "\0");
  fclose(fp);
  return((too_big) ? len-1 : strlen(def-1));
}

// Initialization of static head member
CSgConfig *CSgConfig::head = new CSgConfig;

// Function to write key value to a file
int WritePrivateProfileString(const char* section, const char* key,
                 const char *buf, const char* filename)
{

  FILE *fp;
  int seclen = 0;
  int keylen = 0;
  int buflen = 0;
  int newsec = 0;
  int correct_section = 0;
  int correct_key = 0;
  int section_found = 0;
  int key_found = 0;
  int key_null = 0;
  int file_found = 1;
  char holdsec[512] = "\0";
  char holdkey[512] = "\0";
  char holdval[512] = "\0";
  char holdbuf[512] = "\0";
  CSgConfig *cur;

  if (NULL == filename)
    return 0; // FALSE

  if ((fp = fopen(filename, "r")) == NULL)
    file_found = 0;

  if (NULL == key)
    key_null = 1;

  if (file_found) {
    while(fscanf(fp, "%[^\n]", holdbuf) != EOF) {
      fgetc(fp); // get the newline out of the stream
      buflen = strlen(holdbuf);
      if (0 == buflen)  // blank line
       continue;
      // WE HAVE A SECTION
      if (0 == strncmp(holdbuf, "[", 1)) {
       newsec = 1;
       seclen = strlen(holdbuf);
       strncpy(holdsec, &holdbuf[1], seclen-2);
       strncpy(&holdsec[seclen-2], "\0", 1);
       if (0 == strncmp(holdsec, section, seclen-1)) {
         correct_section = 1;
         section_found = 1;
       }
       else {
         if (correct_section && !key_found) {
           cur = new CSgConfig; // add key to end of section
           strcpy(cur->section, "\0");
           strcpy(cur->key, key);
           strcpy(cur->value, buf);
           strcpy(cur->file, filename);
            key_found = 1; // now that we have added it
         }
         correct_section = 0;
       }
      }
      // WE HAVE A KEY
      else {
       if (key_null && correct_section) {
         key_found = 1;
         continue;
       }
       sscanf(holdbuf, "%[^=]", holdkey);
       strcat(holdkey, "\0");
       keylen = strlen(holdkey);
       if (correct_section && (0 == strncmp(holdkey, key, keylen))) {
         correct_key = 1;
         if (NULL == buf) {
           key_found = 1;
           continue;
         }
       }
       else
         correct_key = 0;
       strncpy(holdval, &holdbuf[keylen+1], (buflen-keylen-1));
       strncpy(&holdval[buflen-keylen-1], "\0", 1);
       cur = new CSgConfig;
       if (newsec)
         strcpy(cur->section, holdsec);
       else
         strcpy(cur->section, "\0");
       strcpy(cur->key, holdkey);
       if (correct_key) {
         strcpy(cur->value, buf);
         key_found = 1;
       }
       else
         strcpy(cur->value, holdval);
       strcpy(cur->file, filename);
       // reinitialize strings
       strcpy(holdkey, "\0");
       newsec = 0;
      }
      strcpy(holdbuf, "\0");
    }
  }

  if (!section_found) { // create a new section for it
    cur = new CSgConfig;
    strcpy(cur->section, section);
    strcpy(cur->key, key);
    strcpy(cur->value, buf);
  }
  if (correct_section && !key_found) {
    cur = new CSgConfig; // add key to section
    strcpy(cur->section, "\0");
    strcpy(cur->key, key);
    strcpy(cur->value, buf);
    strcpy(cur->file, filename);
    key_found = 1; // now that we have added it
  }


  if (file_found)
    fclose(fp);

  if ((fp = fopen(filename, "w")) == NULL)
    return 0; // FALSE

  cur = cur->head;
  cur = cur->next;
  // write all the keys out to the file again
  while (cur != NULL) {
    if (0 != strncmp(cur->section, "\0", 1))
      fprintf(fp, "\n[%s]\n", cur->section);
    fprintf(fp, "%s=%s\n", cur->key, cur->value);
    cur = cur->next;
  }

  fclose(fp);
  cur = cur->remove_list();

  return 1; // TRUE
}
