/*
 * Copyright (c) 1997, Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef _SGLOCK_H_

#define _SGLOCK_H_ 1

#include <stdio.h>
#include <iostream.h>
#include <errno.h>
#include <unistd.h>

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/time.h>

enum { SEM_CLIENT = 0, SEM_SERVER = 1 };
#define MAX_CLIENTS 5 // this is 2 greater than the actual max to
                       // account for the default area

class CSgLock {
private:
  int lockId;
  void init();
#ifdef DEBUG
  void timeout();
#endif

public:
  CSgLock(int key, int server = 0);
  ~CSgLock() {};
  void client_read(int client = -1); // when client wants to read
  void client_finish_read(int client = -1); // when client finishes reading
  void client_write(int client = -1); // when client wants to write
  int client_verify_write(int client = -1); // when client waits for server
  void server_read(int client = -1); // when server wants to read
  void server_finish_read(int client = -1); // when server finishes reading
  void server_write(int client = -1); // when server wants to write
  int server_verify_write(int client = -1); // when server waits for client
  void remove();
};

// SEMAPHORE ARRAY
// [0] -> default client area
// [1] -> default server area
// [2] -> client 1 area
// [3] -> server 1 area
// [4] -> client 2 area
// [5] -> server 2 area
// [6] -> client 3 area
// [7] -> server 3 area
// [8] -> client 4 area
// [9] -> server 4 area

//              CLIENT    SERVER
// INITIAL        1         1
// OK WRITE       1         1
// OK READ        0         0

// CLIENT WRITE   0         -
// CLIENT FINISH  -         1
// CLIENT VERIFY  0/1       -
// SERVER WRITE   -         0
// SERVER FINISH  1         -
// SERVER VERIFY  -         0/1

#endif
