/*
 * Copyright (c) 1997, Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef _SGMESSAGE_H_

#define _SGMESSAGE_H_ 1

#include <iostream.h>

#include <Xm/Xm.h>  // to do the XtAppAddInput stuff
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/systeminfo.h>
#include <sys/filio.h>
#include <stropts.h>
#include <sys/stat.h>
#include <errno.h>

/******************************************************
 *
 *   Class CSgMsg to be used to set up a UNIX socket
 *
 *****************************************************/
enum DataType { SGMSG_CONNECT, SGMSG_READ, SGMSG_EXCEPT,
               SGMSG_CLOSE };

class CSgMsg {
private:
  int listen_sid; // the listening socket if it exists
  int sid;        // the socket id after a connect
  char name[256]; // the name of the UNIX socket
  XtAppContext *app;  // a pointer to the main app context
  XtInputId readId;   // the input id for read events
  XtInputId exceptId; // the input id for exception events
  XtInputId l_readId; // for the listen socket
  XtInputId l_exceptId; // for the listen socket
  XtInputCallbackProc proc; // the process to send the events to
  void (*data_ready)(CSgMsg *, int ); // function to call with data

public:
  // Construct the socket name
  int constr_name(struct sockaddr *addr, const char *pathnm);
  // remove the callbacks from the socket
  int remove_callbacks(int id);
  // listen on the socket
  int listen();
  // check for the existence and remove if needed the socket file
  void check_file(const char *pathnm);

  // return the socket id of the object
  int id() { return sid; }
public:
  // constructor
  CSgMsg(char *pathnm, XtAppContext *context,
        XtInputCallbackProc call);

  // alternate constructor that doesn't setup callbacks
  CSgMsg(char *pathnm, void (*data_func)(CSgMsg *,int));

  // yet one more constructor
  CSgMsg(char *pathnm, int fid);

  // destructor
  ~CSgMsg();

  // create the callbacks on the socket
  int setup_callbacks(int id);

  // bind a name to a socket
  int bind();

  // connect to a socket
  int connect();

  // accept a connection
  CSgMsg * accept();

  // read data from a socket
  int read(void *buf, int len);

  // write data to a socket
  int write(void *buf, int len);

  // close a connection
  int close();

  // helpful functions
  // is_connect is only used by the server side to check for a
  // connect attempt
  int is_connect(int fid, XtInputId xid); // call in Input callback

  int is_read(int fid, XtInputId xid); // call in Input callback

  // is_except checks for an exception on the socket
  int is_except(int fid, XtInputId xid); // call in Input callback

  // always check for is_close before is_read because is_close is a subset
  // of is_read
  int is_close(int fid, XtInputId xid); // call in Input callback

  // server call to check for messages
  int check_for_messages();
};

#endif
