/*
 * Copyright (c) 1999 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdio.h>
#include <string.h>

#include <windows.h>

bool isSGINT()
{
  /*
   * current sgi system strings (add new ones at end of
   *  sgi_sys_ids - rest of code will still function properly)
   * generic pcs default to "AT/AT COMPATIBLE"
   */
  
  const char * sgi_sys_ids[] =
    {
      "SGI-320_ARCx86",
      "SGI-320_ARCx86_mp",
      "SGI-540_ARCx86",
      "SGI-540_ARCx86_mp"
    };
  const char sys_id_reg_entry[] = "HARDWARE\\DESCRIPTION\\System";
  const int n_sgi_sys_ids = sizeof( sgi_sys_ids ) / sizeof( sgi_sys_ids[0] );
  bool is_sgi = false;
  
  /* snork the key string */
    
  unsigned char key_str[ 1024 ]; /* arbitrarily large buffer */
  unsigned long n_key_str = sizeof( key_str );
  DWORD type;
  
  HKEY key = NULL;
  LONG result0 = RegOpenKeyEx( HKEY_LOCAL_MACHINE, sys_id_reg_entry,
			      0, KEY_READ, &key );
  if ( key != NULL )
    {
      LONG result1 = RegQueryValueEx(  key,
				     "Identifier",
				     NULL,
				     &type,
				     key_str,
				     &n_key_str );
      
      /*
       * test the results (default case is already !sgi, so only test
       * on function success)
       */
      if ( ERROR_SUCCESS == result1 )
	{
	  /*
	   * check if one of the tokens matches an sgi string
	   * (could probably get away with just checking that the first 3
	   * characters are SGI, but for correctness, check the whole thing,
	   * for all machine types.
	   */
		 
	  for ( int ii=0; ii<n_sgi_sys_ids; ii++ )
	    {
	      if ( strncmp( sgi_sys_ids[ ii ], (const char *) key_str,
			   strlen( sgi_sys_ids[ ii ] ) ) == 0 )
		{
		  is_sgi = true;
		}
	    }
	}
      
      /* close/return the key */
      RegCloseKey( key );
    }
  
  /* return result */
  return( is_sgi );
}

void main( int argc, char ** argv )
{
  fprintf( stderr, "is sgi nt: %d\n", (int) isSGINT() );
}
